@@ -16,7 +16,10 @@
  *                                                                         *
  ***************************************************************************/
 """
+from __future__ import annotations
+
 import logging
+from typing import TYPE_CHECKING, Optional, Union
 
 from qgis.core import (
     Qgis,
@@ -31,33 +34,45 @@ from qgis.core import (
 )
 from qgis.PyQt.QtCore import QCoreApplication, QSettings
 
-from ..generator.config import IGNORED_FIELDNAMES
+if TYPE_CHECKING:
+    from .fields import Field
+    from .project import Project
+
+from ..generator.config import BASKET_FIELDNAMES, IGNORED_FIELDNAMES
+from ..utils.globals import OptimizeStrategy
 from .form import Form, FormFieldWidget, FormRelationWidget, FormTab
 
 
 class Layer:
     def __init__(
         self,
-        provider=None,
-        uri=None,
-        name=None,
-        srid=None,
-        extent=None,
-        geometry_column=None,
-        wkb_type=QgsWkbTypes.Unknown,
-        alias=None,
-        is_domain=False,
-        is_structure=False,
-        is_nmrel=False,
-        display_expression=None,
-        coordinate_precision=None,
-        is_basket_table=False,
-        is_dataset_table=False,
-        ili_name=None,
-        definitionfile=None,
-        qmlstylefile=None,
-        styles={},
-    ):
+        provider: str = None,
+        uri: str = None,
+        name: str = None,
+        srid: Optional[int] = None,
+        extent: Optional[str] = None,
+        geometry_column: str = None,
+        wkb_type: QgsWkbTypes = QgsWkbTypes.Unknown,
+        alias: Optional[str] = None,
+        is_domain: bool = False,
+        is_structure: bool = False,
+        is_nmrel: bool = False,
+        display_expression: str = None,
+        coordinate_precision: Optional[float] = None,
+        is_basket_table: bool = False,
+        is_dataset_table: bool = False,
+        ili_name: Optional[str] = None,
+        is_relevant: bool = True,
+        all_topics: list[
+            str
+        ] = [],  # all the topics this class (or an instance of it) are located
+        relevant_topics: list[
+            str
+        ] = [],  # the topics of the most extended instance of it only
+        definitionfile: Optional[str] = None,
+        qmlstylefile: Optional[str] = None,
+        styles: dict[str, dict[str, str]] = {},
+    ) -> None:
         self.provider = provider
         self.uri = uri
         self.name = name
@@ -92,12 +107,9 @@ class Layer:
 
         self.ili_name = ili_name
 
-        self.model_topic_name = ""
-        if self.ili_name:
-            if self.ili_name.count(".") > 1:
-                self.model_topic_name = (
-                    f"{self.ili_name.split('.')[0]}.{self.ili_name.split('.')[1]}"
-                )
+        self.is_relevant = is_relevant
+        self.all_topics = all_topics
+        self.relevant_topics = relevant_topics
 
         self.definitionfile = definitionfile
         self.qmlstylefile = qmlstylefile
@@ -110,7 +122,7 @@ class Layer:
         self.checked = True
         self.featurecount = False
 
-    def dump(self):
+    def dump(self) -> dict:
         definition = dict()
         definition["provider"] = self.provider
         definition["uri"] = self.uri
@@ -121,15 +133,17 @@ class Layer:
         definition["isdatasettable"] = self.is_dataset_table
         definition["displayexpression"] = self.display_expression
         definition["coordinateprecision"] = self.coordinate_precision
-        definition["modeltopicname"] = self.model_topic_name
         definition["ili_name"] = self.ili_name
+        definition["is_relevant"] = self.is_relevant
+        definition["all_topics"] = self.all_topics
+        definition["relevant_topics"] = self.relevant_topics
         definition["definitionfile"] = self.definitionfile
         definition["qmlstylefile"] = self.qmlstylefile
         definition["styles"] = self.styles
         definition["form"] = self.__form.dump()
         return definition
 
-    def load(self, definition):
+    def load(self, definition: dict) -> None:
         self.provider = definition["provider"]
         self.uri = definition["uri"]
         self.is_domain = definition["isdomain"]
@@ -139,14 +153,16 @@ class Layer:
         self.is_dataset_table = definition["isdatasettable"]
         self.display_expression = definition["displayexpression"]
         self.coordinate_precision = definition["coordinateprecision"]
-        self.model_topic_name = definition["modeltopicname"]
         self.ili_name = definition["ili_name"]
+        self.is_relevant = definition["is_relevant"]
+        self.all_topics = definition["all_topics"]
+        self.relevant_topics = definition["relevant_topics"]
         self.definitionfile = definition["definitionfile"]
         self.qmlstylefile = definition["qmlstylefile"]
         self.styles = definition["styles"]
         self.__form.load(definition["form"])
 
-    def create(self):
+    def create(self) -> Union[QgsRasterLayer, QgsVectorLayer]:
         if self.definitionfile:
             if self.__layer is None:
                 layers = QgsLayerDefinition.loadLayerDefinitionLayers(
@@ -186,21 +202,16 @@ class Layer:
                 )
                 self.__layer.geometryOptions().setRemoveDuplicateNodes(True)
 
-            if self.model_topic_name:
-                QgsExpressionContextUtils.setLayerVariable(
-                    self.__layer, "interlis_topic", self.model_topic_name
-                )
-
         for field in self.fields:
             field.create(self)
 
         return self.__layer
 
-    def create_form(self, project):
+    def create_form(self, project: Project) -> None:
         edit_form = self.__form.create(self, self.__layer, project)
         self.__layer.setEditFormConfig(edit_form)
 
-    def load_styles(self):
+    def load_styles(self) -> None:
         if self.qmlstylefile:
             self.__layer.loadNamedStyle(self.qmlstylefile)
         if self.styles:
@@ -215,13 +226,32 @@ class Layer:
             # set the default style
             self.__layer.styleManager().setCurrentStyle("default")
 
-    def _create_layer(self, uri, layer_name, provider):
+    def store_variables(self, project: Project) -> None:
+        """
+        Set the layer variables according to the strategy
+        """
+        interlis_topics = ",".join(
+            sorted(self.all_topics)
+            if project.optimize_strategy == OptimizeStrategy.NONE
+            else sorted(self.relevant_topics)
+        )
+        QgsExpressionContextUtils.setLayerVariable(
+            self.__layer, "interlis_topic", interlis_topics
+        )
+
+        QgsExpressionContextUtils.setLayerVariable(
+            self.__layer, "oid_domain", self.oid_domain
+        )
+
+    def _create_layer(
+        self, uri: str, layer_name: str, provider: str
+    ) -> Union[QgsRasterLayer, QgsVectorLayer]:
         if provider and provider.lower() == "wms":
             return QgsRasterLayer(uri, layer_name, provider)
         # return QgsVectorLayer even when it's an invalid layer with no provider
         return QgsVectorLayer(uri, layer_name, provider)
 
-    def post_generate(self, project):
+    def post_generate(self, project: Project) -> None:
         """
         Will be called when the whole project has been generated and
         therefore all relations are available and the form
@@ -250,6 +280,11 @@ class Layer:
             relations_to_add = []
             for relation in project.relations:
                 if relation.referenced_layer == self:
+                    if (
+                        not relation.referencing_layer.is_relevant
+                        and project.optimize_strategy == OptimizeStrategy.GROUP
+                    ):
+                        continue
 
                     # 1:n relation will be added only if does not point to a pure link table
                     if (
@@ -263,13 +298,12 @@ class Layer:
                             if nm_relation == relation:
                                 continue
 
-                            if nm_relation.referenced_layer == self:
+                            if nm_relation.referenced_layer.ili_name == self.ili_name:
                                 continue
 
-                            # relations to the same table with different geometries should not be added
                             if (
-                                self.srid
-                                and nm_relation.referenced_layer.srid == self.srid
+                                not nm_relation.referenced_layer.is_relevant
+                                and project.optimize_strategy == OptimizeStrategy.GROUP
                             ):
                                 continue
 
@@ -293,9 +327,9 @@ class Layer:
                     continue
 
                 if nm_relation:
-                    tab = FormTab(nm_relation.referenced_layer.name)
+                    tab = FormTab(nm_relation.referenced_layer.alias)
                 else:
-                    tab = FormTab(relation.referencing_layer.name)
+                    tab = FormTab(relation.referencing_layer.alias)
 
                 widget = FormRelationWidget(relation, nm_relation)
                 tab.addChild(widget)
@@ -307,15 +341,15 @@ class Layer:
                     widget = FormFieldWidget(field.alias, field.name)
                     self.__form.add_element(widget)
 
-    def source(self):
+    def source(self) -> QgsDataSourceUri:
         return QgsDataSourceUri(self.uri)
 
     @property
-    def layer(self):
+    def layer(self) -> Union[QgsRasterLayer, QgsVectorLayer]:
         return self.__layer
 
     @property
-    def real_id(self):
+    def real_id(self) -> Optional[str]:
         """
         The layer id. Only valid after creating the layer.
         """
@@ -324,7 +358,21 @@ class Layer:
         else:
             return None
 
-    def isPureLinkTable(self, project):
+    @property
+    def oid_domain(self) -> Optional[str]:
+        t_ili_tid_field = self.t_ili_tid_field
+        if t_ili_tid_field:
+            return t_ili_tid_field.oid_domain
+        return None
+
+    @property
+    def t_ili_tid_field(self) -> Optional[Field]:
+        for field in self.fields:
+            if field.name.lower() == "t_ili_tid":
+                return field
+        return None
+
+    def isPureLinkTable(self, project: Project) -> bool:
         """
         Returns True if the layer is a pure link table in a n:m relation.
         With "pure" it is meant the layer has no more fields than foreign keys and its id.
@@ -332,7 +380,7 @@ class Layer:
 
         remaining_fields = set()
         for field in self.fields:
-            if field.name not in IGNORED_FIELDNAMES:
+            if field.name not in IGNORED_FIELDNAMES + BASKET_FIELDNAMES:
                 remaining_fields.add(field.name)
 
         # Remove all fields that are referencing fields
