@@ -16,7 +16,9 @@
  *                                                                         *
  ***************************************************************************/
 """
-from typing import List
+from __future__ import annotations
+
+from typing import Any, Optional
 
 from qgis.core import (
     Qgis,
@@ -33,6 +35,7 @@ from qgis.core import (
 from qgis.PyQt.QtCore import QObject, pyqtSignal
 from qgis.PyQt.QtXml import QDomDocument
 
+from ..utils.globals import OptimizeStrategy
 from .layers import Layer
 from .legend import LegendGroup
 from .relations import Relation
@@ -43,28 +46,35 @@ ENUM_THIS_CLASS_COLUMN = "thisclass"
 class Project(QObject):
     layer_added = pyqtSignal(str)
 
-    def __init__(self, auto_transaction=True, evaluate_default_values=True, context={}):
+    def __init__(
+        self,
+        auto_transaction: bool = True,
+        evaluate_default_values: bool = True,
+        context: dict[str, str] = {},
+        optimize_strategy: OptimizeStrategy = OptimizeStrategy.NONE,
+    ) -> None:
         QObject.__init__(self)
         self.crs = None
         self.name = "Not set"
-        self.layers = List[Layer]
+        self.layers = list[Layer]
         self.legend = LegendGroup()
         self.custom_layer_order_structure = list()
         self.auto_transaction = auto_transaction
         self.evaluate_default_values = evaluate_default_values
-        self.relations = List[Relation]
+        self.relations = list[Relation]
         self.custom_variables = {}
         self.layouts = {}
         self.mapthemes = {}
         self.context = context
+        self.optimize_strategy = optimize_strategy
 
         # {Layer_class_name: {dbattribute: {Layer_class, cardinality, Layer_domain, key_field, value_field]}
         self.bags_of_enum = dict()
 
-    def add_layer(self, layer):
+    def add_layer(self, layer: Layer) -> None:
         self.layers.append(layer)
 
-    def dump(self):
+    def dump(self) -> dict[str, Any]:
         definition = dict()
         definition["crs"] = self.crs.toWkt()
         definition["auto_transaction"] = self.auto_transaction
@@ -88,7 +98,7 @@ class Project(QObject):
 
         return definition
 
-    def load(self, definition):
+    def load(self, definition: dict[str, Any]) -> None:
         self.crs = definition["crs"]
         self.auto_transaction = definition["auto_transaction"]
         self.evaluate_default_values = definition["evaluate_default_values"]
@@ -105,9 +115,26 @@ class Project(QObject):
         self.mapthemes = definition["mapthemes"]
 
     def create(
-        self, path: str, qgis_project: QgsProject, group: QgsLayerTreeGroup = None
-    ):
-        qgis_project.setAutoTransaction(self.auto_transaction)
+        self,
+        path: str,
+        qgis_project: QgsProject,
+        group: Optional[QgsLayerTreeGroup] = None,
+    ) -> None:
+        if Qgis.QGIS_VERSION_INT < 32600:
+            # set auto_transaction as boolean
+            qgis_project.setAutoTransaction(self.auto_transaction)
+        else:
+            # set auto_transaction mode
+            mode = Qgis.TransactionMode.Disabled
+            if (
+                self.auto_transaction == Qgis.TransactionMode.AutomaticGroups.name
+                or self.auto_transaction is True
+            ):
+                mode = Qgis.TransactionMode.AutomaticGroups
+            elif self.auto_transaction == Qgis.TransactionMode.BufferedGroups.name:
+                mode = Qgis.TransactionMode.BufferedGroups
+            qgis_project.setTransactionMode(mode)
+
         qgis_project.setEvaluateDefaultValues(self.evaluate_default_values)
         qgis_layers = list()
         for layer in self.layers:
@@ -157,6 +184,12 @@ class Project(QObject):
                     },
                 )
             elif referenced_layer and referenced_layer.is_basket_table:
+                # list the topics we filter the basket with. On NONE strategy those should be all topics the class could be in. On optimized strategies GROUP/HIDE only the relevant topics should be listed.
+                filter_topics = (
+                    referencing_layer.all_topics
+                    if self.optimize_strategy == OptimizeStrategy.NONE
+                    else referencing_layer.relevant_topics
+                )
                 editor_widget_setup = QgsEditorWidgetSetup(
                     "RelationReference",
                     {
@@ -166,15 +199,18 @@ class Project(QObject):
                         "ShowOpenFormButton": False,
                         "AllowNULL": True,
                         "AllowAddFeatures": False,
-                        "FilterExpression": "\"topic\" = '{}' and attribute(get_feature('{}', 't_id', \"dataset\"), 'datasetname') != '{}'".format(
-                            referencing_layer.model_topic_name,
+                        "FilterExpression": "\"topic\" IN ({}) and attribute(get_feature('{}', 't_id', \"dataset\"), 'datasetname') != '{}'".format(
+                            ",".join(
+                                [f"'{topic}'" for topic in sorted(filter_topics)]
+                            ),  # create comma separated string
                             "T_ILI2DB_DATASET"
                             if referenced_layer.provider == "ogr"
+                            or referenced_layer.provider == "mssql"
                             else "t_ili2db_dataset",
                             self.context.get("catalogue_datasetname", ""),
                         )
-                        if referencing_layer.model_topic_name
-                        else "",
+                        if filter_topics
+                        else "",  # no filter if no topics (means could be everywhere)
                         "FilterFields": list(),
                     },
                 )
@@ -238,7 +274,8 @@ class Project(QObject):
             if layer.layer.type() == QgsMapLayer.VectorLayer:
                 # even when a style will be loaded we create the form because not sure if the style contains form settngs
                 layer.create_form(self)
-                layer.load_styles()
+                layer.store_variables(self)
+            layer.load_styles()
 
         if self.legend:
             self.legend.create(qgis_project, group)
@@ -251,10 +288,12 @@ class Project(QObject):
 
         self.load_layouts(qgis_project)
 
+        self.store_project_variables(qgis_project)
+
         if path:
             qgis_project.write(path)
 
-    def load_custom_layer_order(self, qgis_project):
+    def load_custom_layer_order(self, qgis_project: QgsProject) -> None:
         custom_layer_order = list()
         for custom_layer_name in self.custom_layer_order_structure:
             custom_layer = qgis_project.mapLayersByName(custom_layer_name)
@@ -267,13 +306,13 @@ class Project(QObject):
             root.setCustomLayerOrder(custom_layer_order)
             root.setHasCustomLayerOrder(True)
 
-    def load_custom_variables(self, qgis_project):
+    def load_custom_variables(self, qgis_project: QgsProject) -> None:
         for key in self.custom_variables.keys():
             QgsExpressionContextUtils.setProjectVariable(
                 qgis_project, key, self.custom_variables[key]
             )
 
-    def load_layouts(self, qgis_project):
+    def load_layouts(self, qgis_project: QgsProject) -> None:
         for layout_name in self.layouts.keys():
             # create the layout
             layout = QgsPrintLayout(qgis_project)
@@ -290,7 +329,7 @@ class Project(QObject):
                 layout.setName(layout_name)
                 qgis_project.layoutManager().addLayout(layout)
 
-    def load_mapthemes(self, qgis_project):
+    def load_mapthemes(self, qgis_project: QgsProject) -> None:
         if self.mapthemes:
             for name in self.mapthemes.keys():
                 map_theme_record = QgsMapThemeCollection.MapThemeRecord()
@@ -353,6 +392,11 @@ class Project(QObject):
 
                 qgis_project.mapThemeCollection().insert(name, map_theme_record)
 
-    def post_generate(self):
+    def store_project_variables(self, qgis_project: QgsProject) -> None:
+        QgsExpressionContextUtils.setProjectVariable(
+            qgis_project, "optimize_strategy", self.optimize_strategy.name
+        )
+
+    def post_generate(self) -> None:
         for layer in self.layers:
             layer.post_generate(self)
