@@ -16,6 +16,7 @@
  ***************************************************************************/
 """
 
+import numbers
 import re
 
 import pyodbc
@@ -26,8 +27,9 @@ from .db_connector import DBConnector, DBConnectorError
 METADATA_TABLE = "t_ili2db_table_prop"
 METAATTRS_TABLE = "t_ili2db_meta_attrs"
 SETTINGS_TABLE = "t_ili2db_settings"
-DATASET_TABLE = "t_ili2db_dataset"
-BASKET_TABLE = "t_ili2db_basket"
+DATASET_TABLE = "T_ILI2DB_DATASET"
+BASKET_TABLE = "T_ILI2DB_BASKET"
+NLS_TABLE = "t_ili2db_nls"
 
 
 class MssqlConnector(DBConnector):
@@ -50,6 +52,7 @@ class MssqlConnector(DBConnector):
         self.iliCodeName = "iliCode"
         self.tid = "T_Id"
         self.tilitid = "T_Ili_Tid"
+        self.attachmentKey = "attachmentkey"
         self.dispName = "dispName"
         self.basket_table_name = BASKET_TABLE
         self.dataset_table_name = DATASET_TABLE
@@ -112,6 +115,7 @@ class MssqlConnector(DBConnector):
 
         if self.schema:
             metadata_exists = self.metadata_exists()
+            tr_enabled, lang = self.get_translation_handling()
 
             ln = "\n"
             stmt = ""
@@ -173,6 +177,64 @@ class MssqlConnector(DBConnector):
                 stmt += ln + "    , tgeomtype.setting AS simple_type"
                 stmt += ln + "    , null AS formatted_type"
                 stmt += ln + "    , attrs.sqlname AS attribute_name"
+                stmt += (
+                    ln
+                    + """  , CASE WHEN c.iliname IN (
+                                        SELECT i.baseClass as base
+                                        FROM {schema}.t_ili2db_inheritance i
+                                        LEFT JOIN (
+                                            SELECT fullname, model, topicclass, substring(topicclass, charindex('.',topicclass)+1, len(topicclass)) as class
+                                            FROM (
+                                                SELECT
+                                                    thisClass as fullname,
+                                                    substring(thisClass, 1, charindex('.', thisClass)-1) as model,
+                                                    substring(thisClass, charindex('.', thisClass)+1, len(thisClass)) as topicclass
+                                                FROM {schema}.t_ili2db_inheritance
+                                            ) AS extended
+                                        ) AS extend_names
+                                        ON thisClass = extend_names.fullname
+                                        LEFT JOIN (
+                                            SELECT fullname, model, topicclass, substring(topicclass, charindex('.', topicclass)+1, len(topicclass)) as class
+                                            FROM (
+                                                SELECT
+                                                thisClass as fullname,
+                                                substring(thisClass, 1, charindex('.',thisClass)-1) as model,
+                                                substring(thisClass, charindex('.', thisClass)+1, len(thisClass)) as topicclass
+                                                FROM {schema}.t_ili2db_inheritance
+                                            ) AS topic_level_name
+                                        ) AS base_names
+                                        ON baseClass = base_names.fullname
+                                        -- it's extended
+                                        WHERE baseClass IS NOT NULL
+                                        -- in a different model
+                                        AND base_names.model != extend_names.model
+                                        AND (
+                                            -- with the same name
+                                            base_names.class = extend_names.class
+                                            OR
+                                            -- multiple times in the same extended model
+                                            (SELECT MAX(count) FROM (SELECT COUNT(baseClass) AS count FROM {schema}.t_ili2db_inheritance JOIN (
+                                            SELECT fullname, model, topicclass, substring(topicclass, charindex('.', topicclass)+1, len(topicclass)) as class
+                                            FROM (
+                                                SELECT
+                                                thisClass as fullname,
+                                                substring(thisClass, 1, charindex('.',thisClass)-1) as model,
+                                                substring(thisClass, charindex('.', thisClass)+1, len(thisClass)) as topicclass
+                                                FROM {schema}.t_ili2db_inheritance
+                                            ) AS topic_level_name
+                                        ) AS extend_names ON thisClass = extend_names.fullname WHERE baseClass = i.baseClass GROUP BY baseClass, extend_names.model) AS counts )>1
+                                        )
+                                    )
+                                    THEN 0 ELSE 1 END AS relevance
+                """
+                )
+                stmt += (
+                    ln
+                    + """   ,substring( c.iliname, 1, CHARINDEX('.', substring( c.iliname, CHARINDEX('.', c.iliname)+1, len(c.iliname)))+CHARINDEX('.', c.iliname)-1) as base_topic
+                """
+                )
+            if tr_enabled:
+                stmt += ln + "    , nls.label AS table_tr"
             stmt += ln + "FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS AS Tab"
             stmt += ln + "INNER JOIN INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE AS Col"
             stmt += ln + "    ON Col.Constraint_Name = Tab.Constraint_Name"
@@ -206,6 +268,10 @@ class MssqlConnector(DBConnector):
                 stmt += ln + "    ON tbls.TABLE_NAME = tgeomtype.tablename"
                 stmt += ln + "    AND clm.COLUMN_NAME = tgeomtype.columnname"
                 stmt += ln + "    AND tgeomtype.tag= 'ch.ehi.ili2db.geomType'"
+            if tr_enabled:
+                stmt += ln + "LEFT JOIN {schema}.t_ili2db_nls nls"
+                stmt += ln + "    ON c.iliname = nls.ilielement"
+                stmt += ln + "    AND nls.lang = '{lang}'".format(lang=lang)
             stmt += (
                 ln
                 + "WHERE tbls.TABLE_TYPE = 'BASE TABLE' AND tbls.TABLE_SCHEMA = '{schema}'"
@@ -216,6 +282,7 @@ class MssqlConnector(DBConnector):
                 stmt = self._def_cursor(stmt)
 
             cur = self.conn.cursor()
+
             cur.execute(stmt)
 
             if cur.description:
@@ -227,6 +294,78 @@ class MssqlConnector(DBConnector):
                     my_rec["type"] = my_rec["simple_type"]
                     res.append(my_rec)
 
+            # topics - where this class or an instance of it is located - are emitted by going recursively through the inheritance table.
+            # if something of this topic where the current class is located has been extended, it gets the next child topic.
+            # the relevant topics for optimization are the ones that are not more extended (or in the very last class).
+            # get the topics
+            for res_entry in res:
+                if "base_topic" in res_entry:
+                    stmt = """
+                    WITH children(is_a_base, childTopic, baseTopic) AS (
+                        SELECT
+                        (CASE
+                            WHEN substring( thisClass, 1, CHARINDEX('.', substring( thisClass, CHARINDEX('.', thisClass)+1, len(thisClass)))+CHARINDEX('.', thisClass)-1) IN (SELECT substring( i.baseClass, 1, CHARINDEX('.', substring( i.baseClass, CHARINDEX('.', i.baseClass)+1, len(i.baseClass)))+CHARINDEX('.', i.baseClass)-1) FROM {schema}.T_ILI2DB_INHERITANCE i WHERE substring( i.thisClass, 1,  CHARINDEX('.', i.thisClass) ) != substring( i.baseClass, 1,  CHARINDEX('.', i.baseClass)))
+                            THEN 1
+                            ELSE 0
+                        END) AS is_a_base,
+                        substring( thisClass, 1, CHARINDEX('.', substring( thisClass, CHARINDEX('.', thisClass)+1, len(thisClass)))+CHARINDEX('.', thisClass)-1) as childTopic,
+                        substring( baseClass, 1, CHARINDEX('.', substring( baseClass, CHARINDEX('.', baseClass)+1, len(baseClass)))+CHARINDEX('.', baseClass)-1) as baseTopic
+                        FROM {schema}.T_ILI2DB_INHERITANCE
+                        WHERE substring( baseClass, 1, CHARINDEX('.', substring( baseClass, CHARINDEX('.', baseClass)+1, len(thisClass)))+CHARINDEX('.', baseClass)-1) = '{base_topic}'
+                        UNION ALL
+                        SELECT
+                        (CASE
+                            WHEN substring( thisClass, 1, CHARINDEX('.', substring( thisClass, CHARINDEX('.', thisClass)+1, len(thisClass)))+CHARINDEX('.', thisClass)-1) IN (SELECT substring( i.baseClass, 1, CHARINDEX('.', substring( i.baseClass, CHARINDEX('.', i.baseClass)+1, len(i.baseClass)))+CHARINDEX('.', i.baseClass)-1) FROM {schema}.T_ILI2DB_INHERITANCE i WHERE substring( i.thisClass, 1,  CHARINDEX('.', i.thisClass)) != substring( i.baseClass, 1, CHARINDEX('.', i.baseClass)))
+                            THEN 1
+                            ELSE 0
+                        END) AS is_a_base,
+                        substring( inheritance.thisClass, 1, CHARINDEX('.', substring( inheritance.thisClass, CHARINDEX('.', inheritance.thisClass)+1, len(inheritance.thisClass)))+CHARINDEX('.', inheritance.thisClass)-1) as childTopic ,
+                        substring( inheritance.baseClass, 1, CHARINDEX('.', substring( inheritance.baseClass, CHARINDEX('.', baseClass)+1, len(baseClass)))+CHARINDEX('.', inheritance.baseClass)-1) as baseTopic
+                        FROM children
+                        JOIN {schema}.T_ILI2DB_INHERITANCE as inheritance
+                        ON substring( inheritance.baseClass, 1, CHARINDEX('.', substring( inheritance.baseClass, CHARINDEX('.', inheritance.baseClass)+1, len(inheritance.baseClass)))+CHARINDEX('.', inheritance.baseClass)-1) = children.childTopic -- when the childTopic is as well the baseTopic of another childTopic
+                        WHERE substring( inheritance.thisClass, 1, CHARINDEX('.', substring( inheritance.thisClass, CHARINDEX('.', inheritance.thisClass)+1, len(inheritance.thisClass)))+CHARINDEX('.', inheritance.thisClass)-1) != children.childTopic -- break the recursion when the coming childTopic will be the same
+                    ) SELECT STRING_AGG(childTopic,',') as all_topics FROM children
+                    """.format(
+                        schema=self.schema, base_topic=res_entry["base_topic"]
+                    )
+                    cur.execute(stmt)
+                    res_entry["all_topics"] = cur.fetchone()[0]
+
+            # get the relevant topics
+            for res_entry in res:
+                if "base_topic" in res_entry:
+                    stmt = """
+                    WITH children(is_a_base, childTopic, baseTopic) AS (
+                        SELECT
+                        (CASE
+                            WHEN substring( thisClass, 1, CHARINDEX('.', substring( thisClass, CHARINDEX('.', thisClass)+1, len(thisClass)))+CHARINDEX('.', thisClass)-1) IN (SELECT substring( i.baseClass, 1, CHARINDEX('.', substring( i.baseClass, CHARINDEX('.', i.baseClass)+1, len(i.baseClass)))+CHARINDEX('.', i.baseClass)-1) FROM {schema}.T_ILI2DB_INHERITANCE i WHERE substring( i.thisClass, 1,  CHARINDEX('.', i.thisClass) ) != substring( i.baseClass, 1,  CHARINDEX('.', i.baseClass)))
+                            THEN 1
+                            ELSE 0
+                        END) AS is_a_base,
+                        substring( thisClass, 1, CHARINDEX('.', substring( thisClass, CHARINDEX('.', thisClass)+1, len(thisClass)))+CHARINDEX('.', thisClass)-1) as childTopic,
+                        substring( baseClass, 1, CHARINDEX('.', substring( baseClass, CHARINDEX('.', baseClass)+1, len(baseClass)))+CHARINDEX('.', baseClass)-1) as baseTopic
+                        FROM {schema}.T_ILI2DB_INHERITANCE
+                        WHERE substring( baseClass, 1, CHARINDEX('.', substring( baseClass, CHARINDEX('.', baseClass)+1, len(thisClass)))+CHARINDEX('.', baseClass)-1) = '{base_topic}'
+                        UNION ALL
+                        SELECT
+                        (CASE
+                            WHEN substring( thisClass, 1, CHARINDEX('.', substring( thisClass, CHARINDEX('.', thisClass)+1, len(thisClass)))+CHARINDEX('.', thisClass)-1) IN (SELECT substring( i.baseClass, 1, CHARINDEX('.', substring( i.baseClass, CHARINDEX('.', i.baseClass)+1, len(i.baseClass)))+CHARINDEX('.', i.baseClass)-1) FROM {schema}.T_ILI2DB_INHERITANCE i WHERE substring( i.thisClass, 1,  CHARINDEX('.', i.thisClass)) != substring( i.baseClass, 1, CHARINDEX('.', i.baseClass)))
+                            THEN 1
+                            ELSE 0
+                        END) AS is_a_base,
+                        substring( inheritance.thisClass, 1, CHARINDEX('.', substring( inheritance.thisClass, CHARINDEX('.', inheritance.thisClass)+1, len(inheritance.thisClass)))+CHARINDEX('.', inheritance.thisClass)-1) as childTopic ,
+                        substring( inheritance.baseClass, 1, CHARINDEX('.', substring( inheritance.baseClass, CHARINDEX('.', baseClass)+1, len(baseClass)))+CHARINDEX('.', inheritance.baseClass)-1) as baseTopic
+                        FROM children
+                        JOIN {schema}.T_ILI2DB_INHERITANCE as inheritance
+                        ON substring( inheritance.baseClass, 1, CHARINDEX('.', substring( inheritance.baseClass, CHARINDEX('.', inheritance.baseClass)+1, len(inheritance.baseClass)))+CHARINDEX('.', inheritance.baseClass)-1) = children.childTopic -- when the childTopic is as well the baseTopic of another childTopic
+                        WHERE substring( inheritance.thisClass, 1, CHARINDEX('.', substring( inheritance.thisClass, CHARINDEX('.', inheritance.thisClass)+1, len(inheritance.thisClass)))+CHARINDEX('.', inheritance.thisClass)-1) != children.childTopic -- break the recursion when the coming childTopic will be the same
+                    ) SELECT STRING_AGG(childTopic,',') as relevant_topics FROM children WHERE is_a_base = 0
+                    """.format(
+                        schema=self.schema, base_topic=res_entry["base_topic"]
+                    )
+                    cur.execute(stmt)
+                    res_entry["relevant_topics"] = cur.fetchone()[0]
         return res
 
     def _def_cursor(self, query):
@@ -350,6 +489,7 @@ class MssqlConnector(DBConnector):
         if self.schema:
             metadata_exists = self.metadata_exists()
             metaattrs_exists = self._table_exists(METAATTRS_TABLE)
+            tr_enabled, lang = self.get_translation_handling()
             ln = "\n"
             stmt = ""
 
@@ -375,6 +515,8 @@ class MssqlConnector(DBConnector):
                         + "    , attr_mapping.attr_value AS attr_mapping"
                     )
             stmt += ln + "    , null AS comment"
+            if tr_enabled:
+                stmt += ln + "    , nls.label AS column_tr"
             stmt += ln + "FROM INFORMATION_SCHEMA.COLUMNS AS c"
             if metadata_exists:
                 stmt += ln + "LEFT JOIN {schema}.t_ili2db_column_prop unit"
@@ -407,10 +549,17 @@ class MssqlConnector(DBConnector):
                 if metaattrs_exists:
                     stmt += ln + "LEFT JOIN {schema}.t_ili2db_meta_attrs form_order"
                     stmt += ln + "    ON full_name.iliname=form_order.ilielement AND"
-                    stmt += ln + "    form_order.attr_name='form_order'"
+                    stmt += ln + "    form_order.attr_name IN ("
+                    stmt += ln + "        'form_order',"  # obsolete
+                    stmt += ln + "        'qgis.modelbaker.form_order',"  # obsolete
+                    stmt += ln + "        'qgis.modelbaker.formOrder')"
                     stmt += ln + "LEFT JOIN {schema}.t_ili2db_meta_attrs attr_mapping"
                     stmt += ln + "    ON full_name.iliname=attr_mapping.ilielement AND"
                     stmt += ln + "    attr_mapping.attr_name='ili2db.mapping'"
+                if tr_enabled:
+                    stmt += ln + "LEFT JOIN {schema}.t_ili2db_nls nls"
+                    stmt += ln + "    ON full_name.iliname = nls.ilielement"
+                    stmt += ln + "    AND nls.lang = '{lang}'".format(lang=lang)
             stmt += ln + "WHERE TABLE_NAME = '{table}' AND TABLE_SCHEMA = '{schema}'"
             if metadata_exists and metaattrs_exists:
                 stmt += ln + "ORDER BY attr_order;"
@@ -464,11 +613,36 @@ class MssqlConnector(DBConnector):
 
         return result
 
+    def get_t_type_map_info(self, table_name):
+        if self.schema and self.metadata_exists():
+            cur = self.conn.cursor()
+            cur.execute(
+                """
+                SELECT *
+                FROM {}.t_ili2db_column_prop
+                WHERE tablename = '{}'
+                AND tag = 'ch.ehi.ili2db.types'
+                """.format(
+                    self.schema, table_name
+                )
+            )
+            types_entries = self._get_dict_result(cur)
+
+            types_mapping = dict()
+            for types_entry in types_entries:
+                values = eval(types_entry["setting"])
+                types_mapping[types_entry["columnname"]] = values
+            return types_mapping
+        return {}
+
     def get_relations_info(self, filter_layer_list=[]):
         result = []
 
         if self.schema:
             cur = self.conn.cursor()
+            translate = (
+                ", 1 AS tr_enabled" if self.get_translation_handling()[0] else ""
+            )
             schema_where1 = (
                 "AND KCU1.CONSTRAINT_SCHEMA = '{}'".format(self.schema)
                 if self.schema
@@ -525,6 +699,7 @@ class MssqlConnector(DBConnector):
                     ,KCU1.ORDINAL_POSITION AS ordinal_position
                     {strength_field}
                     {cardinality_max_field}
+                    {translate}
                 FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS AS RC
 
                 INNER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE AS KCU1
@@ -550,6 +725,7 @@ class MssqlConnector(DBConnector):
                 strength_join=strength_join,
                 cardinality_max_field=cardinality_max_field,
                 cardinality_max_join=cardinality_max_join,
+                translate=translate,
             )
             cur.execute(query)
             result = self._get_dict_result(cur)
@@ -582,6 +758,9 @@ class MssqlConnector(DBConnector):
         return result
 
     def get_models(self):
+        if not self._table_exists("t_ili2db_trafo"):
+            return {}
+
         # Get MODELS
         if self.schema:
             cur = self.conn.cursor()
@@ -701,7 +880,6 @@ WHERE TABLE_SCHEMA='{schema}'
         )
 
         res = cur.fetchone()[0]
-        print(res)
         if res > 0:
             self.new_message.emit(
                 Qgis.Warning,
@@ -736,6 +914,7 @@ WHERE TABLE_SCHEMA='{schema}'
                 """SELECT b.t_id as basket_t_id,
                             b.t_ili_tid as basket_t_ili_tid,
                             b.topic as topic,
+                            b.attachmentkey as attachmentkey,
                             d.t_id as dataset_t_id,
                             d.datasetname as datasetname from {schema}.{basket_table} b
                             JOIN {schema}.{dataset_table} d
@@ -781,7 +960,7 @@ WHERE TABLE_SCHEMA='{schema}'
                 return True, self.tr('Successfully created dataset "{}".').format(
                     datasetname
                 )
-            except pyodbc.errors.UniqueViolation as e:
+            except pyodbc.UniqueViolation as e:
                 return False, self.tr('Dataset with name "{}" already exists.').format(
                     datasetname
                 )
@@ -806,7 +985,7 @@ WHERE TABLE_SCHEMA='{schema}'
                 return True, self.tr('Successfully created dataset "{}".').format(
                     datasetname
                 )
-            except pyodbc.errors.UniqueViolation as e:
+            except pyodbc.UniqueViolation as e:
                 return False, self.tr('Dataset with name "{}" already exists.').format(
                     datasetname
                 )
@@ -818,12 +997,15 @@ WHERE TABLE_SCHEMA='{schema}'
             cur = self.conn.cursor()
             cur.execute(
                 """
-                    SELECT DISTINCT PARSENAME(cn.iliname,1) as model,
-                    PARSENAME(cn.iliname,2) as topic
+                    SELECT DISTINCT PARSENAME(cn.iliname,3) as model,
+                    PARSENAME(cn.iliname,2) as topic,
+                    ma.attr_value as bid_domain
                     FROM {schema}.t_ili2db_classname as cn
-                    JOIN {schema}.t_ili2db_table_prop as tp
+                    LEFT JOIN {schema}.t_ili2db_table_prop as tp
                     ON cn.sqlname = tp.tablename
-					WHERE PARSENAME(cn.iliname,3) != '' and tp.setting != 'ENUM'
+                    LEFT JOIN {schema}.t_ili2db_meta_attrs as ma
+                    ON CONCAT(PARSENAME(cn.iliname,3),'.',PARSENAME(cn.iliname,2)) = ma.ilielement AND ma.attr_name = 'ili2db.ili.bidDomain'
+					WHERE PARSENAME(cn.iliname,3) != '' and ( tp.setting != 'ENUM' OR tp.setting IS NULL )
                 """.format(
                     schema=self.schema
                 )
@@ -831,7 +1013,73 @@ WHERE TABLE_SCHEMA='{schema}'
             result = self._get_dict_result(cur)
         return result
 
-    def create_basket(self, dataset_tid, topic):
+    def get_classes_relevance(self):
+        result = []
+        if self.schema and self._table_exists("t_ili2db_classname"):
+            cur = self.conn.cursor()
+            cur.execute(
+                """
+                SELECT
+                    c.iliname as iliname,
+                    c.SqlName as sqlname,
+                    CASE WHEN c.iliname IN (
+                        SELECT i.baseClass as base
+                        FROM {schema}.t_ili2db_inheritance i
+                        LEFT JOIN (
+                            SELECT fullname, model, topicclass, substring(topicclass, charindex('.',topicclass)+1, len(topicclass)) as class
+                            FROM (
+                                SELECT
+                                    thisClass as fullname,
+                                    substring(thisClass, 1, charindex('.', thisClass)-1) as model,
+                                    substring(thisClass, charindex('.', thisClass)+1, len(thisClass)) as topicclass
+                                FROM {schema}.t_ili2db_inheritance
+                            ) AS extended
+                        ) AS extend_names
+                        ON thisClass = extend_names.fullname
+                        LEFT JOIN (
+                            SELECT fullname, model, topicclass, substring(topicclass, charindex('.', topicclass)+1, len(topicclass)) as class
+                            FROM (
+                                SELECT
+                                thisClass as fullname,
+                                substring(thisClass, 1, charindex('.',thisClass)-1) as model,
+                                substring(thisClass, charindex('.', thisClass)+1, len(thisClass)) as topicclass
+                                FROM {schema}.t_ili2db_inheritance
+                            ) AS topic_level_name
+                        ) AS base_names
+                        ON baseClass = base_names.fullname
+                        -- it's extended
+                        WHERE baseClass IS NOT NULL
+                        -- in a different model
+                        AND base_names.model != extend_names.model
+                        AND (
+                            -- with the same name
+                            base_names.class = extend_names.class
+                            OR
+                            -- multiple times in the same extended model
+                            (SELECT MAX(count) FROM (SELECT COUNT(baseClass) AS count FROM {schema}.t_ili2db_inheritance JOIN (
+                            SELECT fullname, model, topicclass, substring(topicclass, charindex('.', topicclass)+1, len(topicclass)) as class
+                            FROM (
+                                SELECT
+                                thisClass as fullname,
+                                substring(thisClass, 1, charindex('.',thisClass)-1) as model,
+                                substring(thisClass, charindex('.', thisClass)+1, len(thisClass)) as topicclass
+                                FROM {schema}.t_ili2db_inheritance
+                            ) AS topic_level_name
+                        ) AS extend_names ON thisClass = extend_names.fullname WHERE baseClass = i.baseClass GROUP BY baseClass, extend_names.model) AS counts )>1
+                        )
+                    )
+                    THEN 0 ELSE 1 END AS relevance
+                FROM {schema}.t_ili2db_classname c
+                """.format(
+                    schema=self.schema
+                )
+            )
+            result = self._get_dict_result(cur)
+        return result
+
+    def create_basket(
+        self, dataset_tid, topic, tilitid_value=None, attachment_key="modelbaker"
+    ):
         if self.schema and self._table_exists(BASKET_TABLE):
             cur = self.conn.cursor()
             cur.execute(
@@ -850,10 +1098,15 @@ WHERE TABLE_SCHEMA='{schema}'
                     topic
                 )
             try:
+                if not tilitid_value:
+                    # default value
+                    tilitid_value = "NEWID()"
+                elif not isinstance(tilitid_value, numbers.Number):
+                    tilitid_value = f"'{tilitid_value}'"
                 cur.execute(
                     """
                     INSERT INTO {schema}.{basket_table} ({tid_name}, dataset, topic, {tilitid_name}, attachmentkey )
-                    VALUES (NEXT VALUE FOR {schema}.{sequence}, {dataset_tid}, '{topic}', NEWID(), 'Qgis Model Baker')
+                    VALUES (NEXT VALUE FOR {schema}.{sequence}, {dataset_tid}, '{topic}', {tilitid}, '{attachment_key}')
                 """.format(
                         schema=self.schema,
                         sequence="t_ili2db_seq",
@@ -862,19 +1115,61 @@ WHERE TABLE_SCHEMA='{schema}'
                         basket_table=BASKET_TABLE,
                         dataset_tid=dataset_tid,
                         topic=topic,
+                        tilitid=tilitid_value,
+                        attachment_key=attachment_key,
                     )
                 )
                 self.conn.commit()
                 return True, self.tr(
                     'Successfully created basket for topic "{}".'
                 ).format(topic)
-            except pyodbc.errors.Error as e:
+            except pyodbc.Error as e:
                 error_message = " ".join(e.args)
                 return False, self.tr(
                     'Could not create basket for topic "{}": {}'
                 ).format(topic, error_message)
         return False, self.tr('Could not create basket for topic "{}".').format(topic)
 
+    def edit_basket(self, basket_config: dict) -> tuple[bool, str]:
+        if self.schema and self._table_exists(BASKET_TABLE):
+            cur = self.conn.cursor()
+            try:
+                cur.execute(
+                    """
+                    UPDATE {schema}.{basket_table}
+                    SET dataset = ?,
+                        {t_ili_tid} = ?,
+                        {attachment_key} = ?
+                    WHERE {tid_name} = ?
+                    """.format(
+                        schema=self.schema,
+                        basket_table=BASKET_TABLE,
+                        t_ili_tid=self.tilitid,
+                        attachment_key=self.attachmentKey,
+                        tid_name=self.tid,
+                    ),
+                    (
+                        basket_config["dataset_t_id"],
+                        basket_config["bid_value"],
+                        basket_config["attachmentkey"],
+                        basket_config["basket_t_id"],
+                    ),
+                )
+                self.conn.commit()
+                return True, self.tr(
+                    'Successfully edited basket for topic "{}" and dataset "{}".'
+                ).format(basket_config["topic"], basket_config["datasetname"])
+            except pyodbc.Error as e:
+                error_message = " ".join(e.args)
+                return False, self.tr(
+                    'Could not edit basket for topic "{}" and dataset "{}": {}'
+                ).format(
+                    basket_config["topic"], basket_config["datasetname"], error_message
+                )
+        return False, self.tr(
+            'Could not edit basket for topic "{}" and dataset "{}"'
+        ).format(basket_config["topic"], basket_config["datasetname"])
+
     def get_tid_handling(self):
         if self.schema and self._table_exists(SETTINGS_TABLE):
             cur = self.conn.cursor()
@@ -905,3 +1200,62 @@ WHERE TABLE_SCHEMA='{schema}'
             )
             result = self._get_dict_result(cur)
         return result
+
+    def get_ili2db_sequence_value(self):
+        # not implemented, return the next one
+        return self.get_next_ili2db_sequence_value()
+
+    def get_next_ili2db_sequence_value(self):
+        if self.schema:
+            cur = self.conn.cursor()
+            cur.execute(
+                """
+                    SELECT NEXT VALUE FOR {schema}.{sequence};
+                """.format(
+                    schema=self.schema, sequence="t_ili2db_seq"
+                )
+            )
+            content = cur.fetchone()
+            if content:
+                return content[0]
+        return None
+
+    def set_ili2db_sequence_value(self, value):
+        if self.schema:
+            cur = self.conn.cursor()
+            try:
+                cur.execute(
+                    """
+                    ALTER SEQUENCE {schema}.{sequence} RESTART WITH {value};
+                    """.format(
+                        schema=self.schema, sequence="t_ili2db_seq", value=value
+                    )
+                )
+                self.conn.commit()
+                return True, self.tr(
+                    'Successfully reset sequence value to "{}".'
+                ).format(value)
+            except pyodbc.Error as e:
+                error_message = " ".join(e.args)
+                return False, self.tr("Could not reset sequence: {}").format(
+                    error_message
+                )
+
+        return False, self.tr("Could not reset sequence")
+
+    def get_translation_handling(self) -> tuple[bool, str]:
+        return self._table_exists(NLS_TABLE) and self._lang != "", self._lang
+
+    def get_available_languages(self):
+        if self.schema and self._table_exists(NLS_TABLE):
+            cur = self.conn.cursor()
+            cur.execute(
+                """
+                SELECT DISTINCT
+                lang
+                FROM {schema}.t_ili2db_nls
+                """
+            ).format(schema=self.schema)
+
+            return [row.lang for row in cur.fetchall()]
+        return []
