@@ -16,7 +16,10 @@
  *                                                                         *
  ***************************************************************************/
 """
+from __future__ import annotations
+
 import re
+from typing import Optional
 
 from qgis.core import QgsApplication, QgsRelation, QgsWkbTypes
 from qgis.PyQt.QtCore import QCoreApplication, QLocale, QObject, pyqtSignal
@@ -26,6 +29,8 @@ from ..dataobjects.layers import Layer
 from ..dataobjects.legend import LegendGroup
 from ..dataobjects.relations import Relation
 from ..db_factory.db_simple_factory import DbSimpleFactory
+from ..iliwrapper.globals import DbIliMode
+from ..utils.globals import OptimizeStrategy
 from ..utils.qt_utils import slugify
 from .config import BASKET_FIELDNAMES, IGNORED_FIELDNAMES, READONLY_FIELDNAMES
 from .domain_relations_generator import DomainRelationGenerator
@@ -39,19 +44,21 @@ class Generator(QObject):
 
     def __init__(
         self,
-        tool,
-        uri,
-        inheritance,
-        schema=None,
-        pg_estimated_metadata=False,
-        parent=None,
-        mgmt_uri=None,
-        consider_basket_handling=False,
-    ):
+        tool: DbIliMode,
+        uri: str,
+        inheritance: str,
+        schema: Optional[str] = None,
+        pg_estimated_metadata: bool = False,
+        parent: QObject = None,
+        mgmt_uri: Optional[str] = None,
+        consider_basket_handling: bool = False,
+        optimize_strategy: OptimizeStrategy = OptimizeStrategy.NONE,
+        preferred_language: str = "",
+    ) -> None:
         """
         Creates a new Generator objects.
         :param uri: The uri that should be used in the resulting project. If authcfg is used, make sure the mgmt_uri is set as well.
-        :param mgmt_uri: The uri that should be used to create schemas, tables and query meta information. Does not support authcfg.
+        :param mgmt_uri: The uri that should be used to create schemas, tables and query meta information. Does not support authcfg but a fallback username.
         :consider_basket_handling: Makes the specific handling of basket tables depending if schema is created with createBasketCol.
         """
         QObject.__init__(self, parent)
@@ -68,6 +75,9 @@ class Generator(QObject):
         self._db_connector.stdout.connect(self.print_info)
         self._db_connector.new_message.connect(self.append_print_message)
         self.basket_handling = consider_basket_handling and self.get_basket_handling()
+        self.optimize_strategy = optimize_strategy
+
+        self._db_connector.set_preferred_translation(preferred_language)
 
         self._additional_ignored_layers = (
             []
@@ -75,21 +85,21 @@ class Generator(QObject):
 
         self.collected_print_messages = []
 
-    def print_info(self, text):
+    def print_info(self, text: str) -> None:
         self.stdout.emit(text)
 
-    def print_messages(self):
+    def print_messages(self) -> None:
         for message in self.collected_print_messages:
             self.new_message.emit(message["level"], message["text"])
         self.collected_print_messages.clear()
 
-    def append_print_message(self, level, text):
+    def append_print_message(self, level, text) -> None:
         message = {"level": level, "text": text}
 
         if message not in self.collected_print_messages:
             self.collected_print_messages.append(message)
 
-    def layers(self, filter_layer_list=[]):
+    def layers(self, filter_layer_list: list = []) -> list[Layer]:
         ignore_basket_tables = not self.basket_handling
         tables_info = self.get_tables_info_without_ignored_tables(ignore_basket_tables)
         layers = list()
@@ -137,7 +147,45 @@ class Generator(QObject):
                 record.get("tablename") == self._db_connector.dataset_table_name
             )
 
-            alias = record["table_alias"] if "table_alias" in record else None
+            is_relevant = bool(
+                record.get("relevance", True)
+            )  # it can be not relevant and still be displayed (in case of NONE)
+
+            base_topic = record["base_topic"] if "base_topic" in record else None
+
+            # get all the topics
+            all_topics = (
+                record.get("all_topics").split(",") if record.get("all_topics") else []
+            )
+            # don't concern no-topics (e.g. when a domain is designed directly in the model)
+            all_topics = [topic for topic in all_topics if topic.count(".") > 0]
+
+            # and include the topic where the class is designed in
+            if base_topic and base_topic.count(".") > 0:
+                all_topics.append(base_topic)
+
+            # get all the relevant topics
+            relevant_topics = (
+                record.get("relevant_topics").split(",")
+                if record.get("relevant_topics")
+                else []
+            )
+
+            # don't concern no-topics (e.g. when a domain is designed directly in the model)
+            relevant_topics = [
+                topic for topic in relevant_topics if topic.count(".") > 0
+            ]
+
+            # if no relevant_topic found the relevant is the one where the class is designed in
+            if not relevant_topics and base_topic and base_topic.count(".") > 0:
+                relevant_topics.append(base_topic)
+
+            # Get table name in this order: translation if exists,
+            # alias (dispName) if exists, or ili_name.
+            alias = record.get("table_tr", None)
+            if not alias:
+                alias = record.get("table_alias", None)
+
             if not alias:
                 short_name = None
                 if is_domain and is_attribute:
@@ -153,7 +201,7 @@ class Generator(QObject):
                         table_appearance_count[record["tablename"]] > 1
                         and "geometry_column" in record
                     ):
-                        # multiple layers for this table - append geometry column to name
+                        # table loaded multiple times (because of multiple geometry columns) - append geometry column to name (for PG source layers)
                         fields_info = self.get_fields_info(record["tablename"])
                         for field_info in fields_info:
                             if field_info["column_name"] == record["geometry_column"]:
@@ -178,7 +226,7 @@ class Generator(QObject):
                         if match.group(0) == match.group(1):
                             short_name = match.group(1).split(".")[-1]
                         else:
-                            # additional brackets in the the name - extended layer in geopackage
+                            # table does not fit in match group (on multigeometry in geopackage they are named with "...OriginalName.GeometryColumn (OriginalName)" so we want the GeometryColumn appended in brackets
                             short_name = (
                                 match.group(1).split(".")[-2]
                                 + " ("
@@ -197,7 +245,10 @@ class Generator(QObject):
             elif "ili_name" in record and record["ili_name"]:
                 meta_attrs = self.get_meta_attrs(record["ili_name"])
                 for attr_record in meta_attrs:
-                    if attr_record["attr_name"] == "dispExpression":
+                    if attr_record["attr_name"] in [
+                        "dispExpression",  # obsolete
+                        "qgis.modelbaker.dispExpression",
+                    ]:
                         display_expression = attr_record["attr_value"]
 
             coord_decimals = (
@@ -207,6 +258,11 @@ class Generator(QObject):
             if coord_decimals:
                 coordinate_precision = 1 / (10**coord_decimals)
 
+            layer_uri.gpkg_multigeom = bool(
+                table_appearance_count[record["tablename"]] > 1
+                and "geometry_column" in record
+            )
+
             layer = Layer(
                 layer_uri.provider,
                 layer_uri.get_data_source_uri(record),
@@ -224,32 +280,42 @@ class Generator(QObject):
                 is_basket_table,
                 is_dataset_table,
                 record.get("ili_name"),
+                is_relevant,
+                all_topics,
+                relevant_topics,
             )
 
             # Configure fields for current table
             fields_info = self.get_fields_info(record["tablename"])
             min_max_info = self.get_min_max_info(record["tablename"])
+            # We get the value map values from the check-constraints (only pg support) and additionally we check the t_type values in the ili2db-meta-tables
             value_map_info = self.get_value_map_info(record["tablename"])
-            re_iliname = re.compile(r".*\.(.*)$")
+            t_type_map_info = self.get_t_type_map_info(record["tablename"])
+            value_map_info.update(t_type_map_info)
 
+            re_iliname = re.compile(r".*\.(.*)$")
             for fielddef in fields_info:
                 column_name = fielddef["column_name"]
-                fully_qualified_name = (
-                    fielddef["fully_qualified_name"]
-                    if "fully_qualified_name" in fielddef
-                    else None
-                )
-                m = (
-                    re_iliname.match(fully_qualified_name)
-                    if fully_qualified_name
-                    else None
-                )
 
-                alias = None
-                if "column_alias" in fielddef:
-                    alias = fielddef["column_alias"]
-                if m and not alias:
-                    alias = m.group(1)
+                # Get field name in this order: translation if exists,
+                # alias (dispName) if exists, or ili_name.
+                alias = fielddef.get("column_tr", None)
+                if not alias:
+                    alias = fielddef.get("column_alias", None)
+
+                if not alias:
+                    fully_qualified_name = (
+                        fielddef["fully_qualified_name"]
+                        if "fully_qualified_name" in fielddef
+                        else None
+                    )
+                    m = (
+                        re_iliname.match(fully_qualified_name)
+                        if fully_qualified_name
+                        else None
+                    )
+                    if m:
+                        alias = m.group(1)
 
                 field = Field(column_name)
                 field.alias = alias
@@ -295,9 +361,25 @@ class Generator(QObject):
 
                 if column_name in value_map_info:
                     field.widget = "ValueMap"
-                    field.widget_config["map"] = [
-                        {val: val} for val in value_map_info[column_name]
-                    ]
+
+                    # if it's the t_type column, then handle concerning relevance
+                    if column_name.lower() == "t_type":
+                        tables_relevance_info = self.get_tables_relevance()
+                        field.widget_config["map"] = []
+                        for val in value_map_info[column_name]:
+                            if (
+                                not tables_relevance_info[val]["relevance"]
+                                and self.optimize_strategy == OptimizeStrategy.HIDE
+                            ):
+                                continue
+                            field.widget_config["map"].append(
+                                {tables_relevance_info[val]["iliname"]: val}
+                            )
+                            field.default_value_expression = f"'{val}'"
+                    else:
+                        field.widget_config["map"] = [
+                            {val: val} for val in value_map_info[column_name]
+                        ]
 
                 if "attr_mapping" in fielddef and fielddef["attr_mapping"] == "ARRAY":
                     field.widget = "List"
@@ -330,33 +412,85 @@ class Generator(QObject):
 
                 db_factory.customize_widget_editor(field, data_type)
 
+                if "enum_domain" in fielddef and fielddef["enum_domain"]:
+                    field.enum_domain = fielddef["enum_domain"]
+
+                # default value expressions
+
+                ## we have this to provide e.g. the T_Id expression for GPKG defined in the db_connector
                 if "default_value_expression" in fielddef:
                     field.default_value_expression = fielddef[
                         "default_value_expression"
                     ]
 
+                ## oid (t_ili_tid)
+                # trying to set reasonable default default-expressions:
+                if column_name == self._db_connector.tilitid:
+                    field.oid_domain = fielddef.get("oid_domain", None)
+                    if field.oid_domain == "INTERLIS.UUIDOID":
+                        # clear case
+                        field.default_value_expression = "uuid('WithoutBraces')"
+                    elif field.oid_domain == "INTERLIS.I32OID":
+                        # taking the tid as stable serial
+                        field.default_value_expression = self._db_connector.tid
+                    elif field.oid_domain == "INTERLIS.STANDARDOID":
+                        # taking the example prefix + the tid as stable serial in 8 chars
+                        field.default_value_expression = (
+                            f"'ch100000' || lpad( {self._db_connector.tid}, 8, 0 )"
+                        )
+                    else:
+                        # ANY, user- or not-defined (mostly OID TEXT, means no leading digits allowed)
+                        field.default_value_expression = "'_' || uuid('WithoutBraces')"
+
+                ## basket (t_basket)
                 if self.basket_handling and column_name in BASKET_FIELDNAMES:
+                    # on NONE strategy those should be all topics the class could be in. On optimized strategies GROUP/HIDE only the relevant topics should be listed.
+                    interlis_topics = ",".join(
+                        sorted(layer.all_topics)
+                        if self.optimize_strategy == OptimizeStrategy.NONE
+                        else sorted(layer.relevant_topics)
+                    )
+
+                    # and set the default value (to be used from the projet variables)
                     default_basket_topic = slugify(
-                        f"default_basket{'_' if layer.model_topic_name else ''}{layer.model_topic_name}"
+                        f"default_basket{'_' if interlis_topics else ''}{interlis_topics}"
                     )
                     field.default_value_expression = f"@{default_basket_topic}"
 
-                if column_name == self._db_connector.tilitid:
-                    # when there is a t_ili_tid it should be filled up when there is no OID defined in the model
-                    if "oid_domain" not in fielddef or fielddef["oid_domain"] is None:
-                        field.default_value_expression = "substr(uuid(), 2, 36)"
-
-                if "enum_domain" in fielddef and fielddef["enum_domain"]:
-                    field.enum_domain = fielddef["enum_domain"]
-
                 layer.fields.append(field)
 
             layers.append(layer)
 
+        # append topic name to ambiguous layers
+        self._rename_ambiguous_layers(layers)
+        # append model name to still ambiguous layers
+        self._rename_ambiguous_layers(layers, second_pass=True)
+
         self.print_messages()
 
         return layers
 
+    def _rename_ambiguous_layers(self, layers, second_pass=False):
+        # rename ambiguous layers with topic (on not second_pass) or model (on second_pass) prefix
+        # on irrelevant layers only if we don't ride OptimizeStrategy.HIDE or we do but on smart1
+        aliases = [
+            l.alias
+            for l in layers
+            if l.is_relevant
+            or self.optimize_strategy != OptimizeStrategy.HIDE
+            or self.inheritance == "smart1"
+        ]
+        ambiguous_aliases = [alias for alias in aliases if aliases.count(alias) > 1]
+        for layer in layers:
+            if layer.alias in ambiguous_aliases:
+                if layer.ili_name:
+                    if layer.ili_name.count(".") > 1:
+                        layer.alias = (
+                            layer.ili_name.split(".")[(0 if second_pass else 1)]
+                            + "."
+                            + layer.alias
+                        )
+
     def relations(self, layers, filter_layer_list=[]):
         relations_info = self.get_relations_info(filter_layer_list)
         layer_map = dict()
@@ -376,17 +510,31 @@ class Generator(QObject):
                 and record["referenced_table"] in layer_map.keys()
             ):
                 for referencing_layer in layer_map[record["referencing_table"]]:
+                    if (
+                        not referencing_layer.is_relevant
+                        and self.optimize_strategy == OptimizeStrategy.HIDE
+                        and self.inheritance == "smart2"
+                    ):
+                        continue
                     for referenced_layer in layer_map[record["referenced_table"]]:
+                        if (
+                            not referenced_layer.is_relevant
+                            and self.optimize_strategy == OptimizeStrategy.HIDE
+                            and self.inheritance == "smart2"
+                        ):
+                            continue
                         relation = Relation()
                         relation.referencing_layer = referencing_layer
                         relation.referenced_layer = referenced_layer
                         relation.referencing_field = record["referencing_column"]
                         relation.referenced_field = record["referenced_column"]
                         relation.name = record["constraint_name"]
+                        relation.translate_name = record.get("tr_enabled", False)
                         relation.strength = (
                             QgsRelation.Composition
                             if "strength" in record
                             and record["strength"] == "COMPOSITE"
+                            or referencing_layer.is_structure
                             else QgsRelation.Association
                         )
                         relation.cardinality_max = record.get("cardinality_max", None)
@@ -578,35 +726,28 @@ class Generator(QObject):
                 if node:
                     legend.append(node)
         else:
-            tables = LegendGroup(QCoreApplication.translate("LegendGroup", "tables"))
-            domains = LegendGroup(QCoreApplication.translate("LegendGroup", "domains"))
-            domains.expanded = False
-            system = LegendGroup(QCoreApplication.translate("LegendGroup", "system"))
-            system.expanded = False
-
-            point_layers = []
-            line_layers = []
-            polygon_layers = []
-
-            for layer in layers:
-                if layer.geometry_column:
-                    geometry_type = QgsWkbTypes.geometryType(layer.wkb_type)
-                    if geometry_type == QgsWkbTypes.PointGeometry:
-                        point_layers.append(layer)
-                    elif geometry_type == QgsWkbTypes.LineGeometry:
-                        line_layers.append(layer)
-                    elif geometry_type == QgsWkbTypes.PolygonGeometry:
-                        polygon_layers.append(layer)
-                else:
-                    if layer.is_domain:
-                        domains.append(layer)
-                    elif layer.name in [
-                        self._db_connector.basket_table_name,
-                        self._db_connector.dataset_table_name,
-                    ]:
-                        system.append(layer)
+            irrelevant_layers = []
+            relevant_layers = []
+            if (
+                self.optimize_strategy == OptimizeStrategy.NONE
+                or self.inheritance == "smart1"
+            ):
+                relevant_layers = layers
+            else:
+                for layer in layers:
+                    if layer.is_relevant:
+                        relevant_layers.append(layer)
                     else:
-                        tables.append(layer)
+                        irrelevant_layers.append(layer)
+
+            (
+                point_layers,
+                line_layers,
+                polygon_layers,
+                domain_layers,
+                table_layers,
+                system_layers,
+            ) = self._separated_legend_layers(relevant_layers)
 
             for l in polygon_layers:
                 legend.append(l)
@@ -615,15 +756,115 @@ class Generator(QObject):
             for l in point_layers:
                 legend.append(l)
 
-            if not tables.is_empty():
+            # create groups
+            if len(table_layers):
+                tables = LegendGroup(
+                    QCoreApplication.translate("LegendGroup", "tables")
+                )
+                for layer in table_layers:
+                    tables.append(layer)
                 legend.append(tables)
-            if not domains.is_empty():
+            if len(domain_layers):
+                domains = LegendGroup(
+                    QCoreApplication.translate("LegendGroup", "domains")
+                )
+                domains.expanded = False
+                for layer in domain_layers:
+                    domains.append(layer)
                 legend.append(domains)
-            if not system.is_empty():
+            if len(system_layers):
+                system = LegendGroup(
+                    QCoreApplication.translate("LegendGroup", "system")
+                )
+                system.expanded = False
+                for layer in system_layers:
+                    system.append(layer)
                 legend.append(system)
 
+            # when the irrelevant layers should be grouped (but visible), we make the structure for them and append it to a group
+            if self.optimize_strategy == OptimizeStrategy.GROUP:
+                (
+                    point_layers,
+                    line_layers,
+                    polygon_layers,
+                    domain_layers,
+                    table_layers,
+                    system_layers,
+                ) = self._separated_legend_layers(irrelevant_layers)
+
+                # create base group
+                base_group = LegendGroup(
+                    QCoreApplication.translate("LegendGroup", "base layers")
+                )
+                base_group.expanded = False
+                base_group.checked = False
+
+                for l in polygon_layers:
+                    base_group.append(l)
+                for l in line_layers:
+                    base_group.append(l)
+                for l in point_layers:
+                    base_group.append(l)
+
+                # create groups
+                if len(table_layers):
+                    tables = LegendGroup(
+                        QCoreApplication.translate("LegendGroup", "base tables")
+                    )
+                    for layer in table_layers:
+                        tables.append(layer)
+                    base_group.append(tables)
+                if len(domain_layers):
+                    domains = LegendGroup(
+                        QCoreApplication.translate("LegendGroup", "base domains")
+                    )
+                    domains.expanded = False
+                    for layer in domain_layers:
+                        domains.append(layer)
+                    base_group.append(domains)
+
+                legend.append(base_group)
+
         return legend
 
+    def _separated_legend_layers(self, layers):
+        domain_layers = []
+        table_layers = []
+        system_layers = []
+
+        point_layers = []
+        line_layers = []
+        polygon_layers = []
+
+        for layer in layers:
+            if layer.geometry_column:
+                geometry_type = QgsWkbTypes.geometryType(layer.wkb_type)
+                if geometry_type == QgsWkbTypes.PointGeometry:
+                    point_layers.append(layer)
+                elif geometry_type == QgsWkbTypes.LineGeometry:
+                    line_layers.append(layer)
+                elif geometry_type == QgsWkbTypes.PolygonGeometry:
+                    polygon_layers.append(layer)
+            else:
+                if layer.is_domain:
+                    domain_layers.append(layer)
+                elif layer.name in [
+                    self._db_connector.basket_table_name,
+                    self._db_connector.dataset_table_name,
+                ]:
+                    system_layers.append(layer)
+                else:
+                    table_layers.append(layer)
+
+        return (
+            point_layers,
+            line_layers,
+            polygon_layers,
+            domain_layers,
+            table_layers,
+            system_layers,
+        )
+
     def resolved_layouts(
         self,
         layouts={},
@@ -686,6 +927,22 @@ class Generator(QObject):
     def get_value_map_info(self, table_name):
         return self._db_connector.get_value_map_info(table_name)
 
+    def get_tables_relevance(self):
+        """
+        Returns a dict with relevance info per sqlname of class.
+        """
+        tables_relevance = {}
+        for record in self._db_connector.get_classes_relevance():
+            tables_relevance[record["sqlname"]] = {
+                "iliname": record["iliname"],
+                "relevance": record["relevance"],
+            }
+
+        return tables_relevance
+
+    def get_t_type_map_info(self, table_name):
+        return self._db_connector.get_t_type_map_info(table_name)
+
     def get_relations_info(self, filter_layer_list=[]):
         return self._db_connector.get_relations_info(filter_layer_list)
 
