@@ -20,6 +20,7 @@
 from qgis.core import QgsNetworkAccessManager
 from qgis.PyQt.QtNetwork import QNetworkProxy
 
+from .globals import DbIliMode
 from .ili2dbutils import get_all_modeldir_in_path
 
 
@@ -101,7 +102,7 @@ class BaseConfiguration:
         return dirs
 
     @property
-    def metaconfig_directories(self):
+    def ilidata_directories(self):
         dirs = list()
         if self.custom_model_directories_enabled and self.custom_model_directories:
             dirs = self.custom_model_directories.split(";")
@@ -111,31 +112,35 @@ class BaseConfiguration:
 
 
 class Ili2DbCommandConfiguration:
-    def __init__(self):
-        self.base_configuration = BaseConfiguration()
+    def __init__(self, other=None):
+        if not isinstance(other, Ili2DbCommandConfiguration):
+            self.base_configuration = BaseConfiguration()
 
-        self.dbport = ""
-        self.dbhost = ""
-        self.dbpwd = ""
-        self.dbusr = ""
-        self.dbauthid = ""
-        self.db_use_super_login = False
-        self.database = ""
-        self.dbschema = ""
-        self.dbfile = ""
-        self.dbservice = None
-        self.sslmode = None
-        self.tool = None
-        self.ilifile = ""
-        self.ilimodels = ""
-        self.tomlfile = ""
-        self.dbinstance = ""
-        self.db_odbc_driver = ""
-        self.disable_validation = False
-        self.metaconfig = None
-        self.metaconfig_id = None
-        self.metaconfig_params_only = False
-        self.db_ili_version = None
+            self.dbport = ""
+            self.dbhost = ""
+            self.dbpwd = ""
+            self.dbusr = ""
+            self.dbauthid = ""
+            self.db_use_super_login = False
+            self.database = ""
+            self.dbschema = ""
+            self.dbfile = ""
+            self.dbservice = None
+            self.sslmode = None
+            self.tool = None
+            self.ilifile = ""
+            self.ilimodels = ""
+            self.tomlfile = ""
+            self.dbinstance = ""
+            self.db_odbc_driver = ""
+            self.disable_validation = False
+            self.metaconfig = None
+            self.metaconfig_id = None
+            self.metaconfig_params_only = False
+            self.db_ili_version = None
+        else:
+            # We got an 'other' object from which we'll get parameters
+            self.__dict__ = other.__dict__.copy()
 
     def append_args(self, args, values, consider_metaconfig=False, force_append=False):
 
@@ -186,8 +191,8 @@ class Ili2DbCommandConfiguration:
 
 
 class ExportConfiguration(Ili2DbCommandConfiguration):
-    def __init__(self):
-        super().__init__()
+    def __init__(self, other: Ili2DbCommandConfiguration = None):
+        super().__init__(other)
         self.xtffile = ""
         self.with_exporttid = False
         self.iliexportmodels = ""
@@ -228,13 +233,14 @@ class ExportConfiguration(Ili2DbCommandConfiguration):
 
 
 class SchemaImportConfiguration(Ili2DbCommandConfiguration):
-    def __init__(self):
-        super().__init__()
+    def __init__(self, other: Ili2DbCommandConfiguration = None):
+        super().__init__(other)
         self.inheritance = "smart1"
         self.create_basket_col = False
         self.create_import_tid = True
         self.srs_auth = "EPSG"  # Default SRS auth in ili2db
         self.srs_code = 2056  # Default SRS code in ili2db
+        self.create_gpkg_multigeom = False
         self.stroke_arcs = True
         self.pre_script = ""
         self.post_script = ""
@@ -252,7 +258,6 @@ class SchemaImportConfiguration(Ili2DbCommandConfiguration):
         self.append_args(args, extra_args, force_append=True)
 
         self.append_args(args, ["--coalesceCatalogueRef"], True)
-        self.append_args(args, ["--createEnumTabs"], True)
 
         if self.disable_validation:
             self.append_args(args, ["--sqlEnableNull"], force_append=True)
@@ -276,10 +281,11 @@ class SchemaImportConfiguration(Ili2DbCommandConfiguration):
             self.append_args(args, ["--createTypeConstraint"], True)
             self.append_args(args, ["--createEnumTabsWithId"], True)
             self.append_args(args, ["--createTidCol"], True)
-
-        # version 3 backwards compatibility (not needed in newer versions)
-        if self.create_import_tid:
-            self.append_args(args, ["--importTid"])
+        else:
+            # version 3 backwards compatibility (not needed in newer versions)
+            self.append_args(args, ["--createEnumTabs"], True)
+            if self.create_import_tid:
+                self.append_args(args, ["--importTid"])
 
         if self.inheritance == "smart1":
             self.append_args(args, ["--smart1Inheritance"])
@@ -293,6 +299,12 @@ class SchemaImportConfiguration(Ili2DbCommandConfiguration):
         elif self.db_ili_version is None or self.db_ili_version > 3:
             self.append_args(args, ["--strokeArcs=False"])
 
+        if self.tool and (self.tool & DbIliMode.gpkg):
+            if self.create_gpkg_multigeom:
+                self.append_args(args, ["--gpkgMultiGeomPerTable"], True)
+            elif self.db_ili_version is None or self.db_ili_version > 3:
+                self.append_args(args, ["--gpkgMultiGeomPerTable=False"])
+
         if self.create_basket_col:
             self.append_args(args, ["--createBasketCol"])
         elif self.db_ili_version is None or self.db_ili_version > 3:
@@ -312,6 +324,9 @@ class SchemaImportConfiguration(Ili2DbCommandConfiguration):
         elif self.db_ili_version is None or self.db_ili_version > 3:
             self.append_args(args, ["--postScript", "NULL"])
 
+        if self.db_ili_version is None or self.db_ili_version > 3:
+            self.append_args(args, ["--createNlsTab"])
+
         self.append_args(
             args, Ili2DbCommandConfiguration.to_ili2db_args(self), force_append=True
         )
@@ -323,8 +338,8 @@ class SchemaImportConfiguration(Ili2DbCommandConfiguration):
 
 
 class ImportDataConfiguration(SchemaImportConfiguration):
-    def __init__(self):
-        super().__init__()
+    def __init__(self, other: Ili2DbCommandConfiguration = None):
+        super().__init__(other)
         self.xtffile = ""
         self.delete_data = False
         self.with_importtid = False
@@ -376,10 +391,11 @@ class ImportDataConfiguration(SchemaImportConfiguration):
 
 
 class UpdateDataConfiguration(Ili2DbCommandConfiguration):
-    def __init__(self):
-        super().__init__()
+    def __init__(self, other: Ili2DbCommandConfiguration = None):
+        super().__init__(other)
         self.xtffile = ""
         self.dataset = ""
+        self.delete_data = False
         self.with_importtid = False
         self.with_importbid = False
 
@@ -387,7 +403,10 @@ class UpdateDataConfiguration(Ili2DbCommandConfiguration):
         args = list()
 
         if with_action:
-            self.append_args(args, ["--update"])
+            if self.delete_data:
+                self.append_args(args, ["--replace"])
+            else:
+                self.append_args(args, ["--update"])
 
         self.append_args(args, extra_args)
 
@@ -410,8 +429,8 @@ class UpdateDataConfiguration(Ili2DbCommandConfiguration):
 
 
 class ValidateConfiguration(Ili2DbCommandConfiguration):
-    def __init__(self):
-        super().__init__()
+    def __init__(self, other: Ili2DbCommandConfiguration = None):
+        super().__init__(other)
         self.ilimodels = ""
         self.topics = ""
         self.dataset = ""
@@ -421,6 +440,7 @@ class ValidateConfiguration(Ili2DbCommandConfiguration):
         self.xtflog = ""
         self.skip_geometry_errors = False
         self.valid_config = ""
+        self.xtffile = ""
 
     def to_ili2db_args(self, extra_args=[], with_action=True):
         args = list()
@@ -448,6 +468,9 @@ class ValidateConfiguration(Ili2DbCommandConfiguration):
         if self.with_exporttid:
             self.append_args(args, ["--exportTid"])
 
+        if self.db_ili_version == 3:
+            self.append_args(args, ["--export3"])
+
         if self.xtflog:
             self.append_args(args, ["--xtflog", self.xtflog])
 
@@ -458,6 +481,34 @@ class ValidateConfiguration(Ili2DbCommandConfiguration):
         if self.valid_config:
             self.append_args(args, ["--validConfig", self.valid_config])
 
+        if self.db_ili_version == 3:
+            self.append_args(args, [self.xtffile])
+
+        self.append_args(args, Ili2DbCommandConfiguration.to_ili2db_args(self))
+
+        return args
+
+
+class DeleteConfiguration(Ili2DbCommandConfiguration):
+    def __init__(self, other: Ili2DbCommandConfiguration = None):
+        super().__init__(other)
+        self.dataset = ""
+        self.baskets = ""
+
+    def to_ili2db_args(self, extra_args=[], with_action=True):
+        args = list()
+
+        if with_action:
+            self.append_args(args, ["--delete"])
+
+        self.append_args(args, extra_args)
+
+        if self.dataset:
+            self.append_args(args, ["--dataset", self.dataset])
+
+        if self.baskets:
+            self.append_args(args, ["--baskets", self.baskets])
+
         self.append_args(args, Ili2DbCommandConfiguration.to_ili2db_args(self))
 
         return args
