@@ -20,11 +20,12 @@ import glob
 import logging
 import os
 import re
+import shutil
 import urllib.parse
 import xml.etree.ElementTree as ET
 from enum import Enum
 
-from PyQt5.QtCore import QObject, Qt, pyqtSignal
+from PyQt5.QtCore import QDir, QObject, QSortFilterProxyModel, Qt, QTimer, pyqtSignal
 from PyQt5.QtGui import QPalette, QRegion, QStandardItem, QStandardItemModel
 from PyQt5.QtWidgets import QGridLayout, QItemDelegate, QLabel, QStyle, QWidget
 from qgis.core import Qgis, QgsMessageLog
@@ -38,18 +39,33 @@ class IliCache(QObject):
 
     new_message = pyqtSignal(int, str)
 
+    CACHE_PATH = os.path.expanduser("~/.ilicache")
+
     def __init__(self, configuration, single_ili_file=None):
         QObject.__init__(self)
-        self.cache_path = os.path.expanduser("~/.ilicache")
         self.information_file = "ilimodels.xml"
         self.repositories = dict()
         self.base_configuration = configuration
         self.single_ili_file = single_ili_file
         self.model = IliModelItemModel()
+        self.sorted_model = QSortFilterProxyModel()
+        self.sorted_model.setSourceModel(self.model)
+        self.sorted_model.sort(0, Qt.AscendingOrder)
         self.directories = None
         if self.base_configuration:
             self.directories = self.base_configuration.model_directories
 
+        # refresh the models on changing values but avoid massive db connects by timer
+        self.modelReposTimer = QTimer()
+        self.modelReposTimer.setSingleShot(True)
+        self.modelReposTimer.timeout.connect(
+            lambda: self.model.set_repositories(self.repositories)
+        )
+
+    def set_repositories_to_model(self):
+        # hold refresh back
+        self.modelReposTimer.start(500)
+
     def refresh(self):
         if not self.directories is None:
             for directory in self.directories:
@@ -75,7 +91,7 @@ class IliCache(QObject):
         self.repositories["no_repo"] = sorted(
             models, key=lambda m: m["version"], reverse=True
         )
-        self.model.set_repositories(self.repositories)
+        self.set_repositories_to_model()
 
     def file_url(self, url, file):
         if url is None:
@@ -92,14 +108,15 @@ class IliCache(QObject):
             return os.path.join(url, file)
         else:
             netloc = "" if netloc is None else netloc
-            return os.path.join(self.cache_path, netloc, file)
+            return os.path.join(self.CACHE_PATH, netloc, file)
 
     def download_repository(self, url):
         """
         Downloads the informationfile (default: ilimodels.xml) and ilisite.xml files from the provided url
         and updates the local cache.
         """
-        netloc = urllib.parse.urlsplit(url)[1] if not os.path.isdir(url) else url
+        parsed_url = urllib.parse.urlparse(url)
+        netloc = parsed_url.netloc + parsed_url.path if not os.path.isdir(url) else url
 
         information_file_url = self.file_url(url, self.information_file)
         ilisite_url = self.file_url(url, "ilisite.xml")
@@ -120,10 +137,10 @@ class IliCache(QObject):
                     self.tr("Could not find local file  {}").format(ilisite_url)
                 )
         else:
-            netloc_dir = os.path.join(self.cache_path, netloc)
+            netloc_dir = os.path.join(self.CACHE_PATH, netloc)
             os.makedirs(netloc_dir, exist_ok=True)
             information_file_path = os.path.join(netloc_dir, self.information_file)
-            ilisite_path = os.path.join(self.cache_path, netloc, "ilisite.xml")
+            ilisite_path = os.path.join(self.CACHE_PATH, netloc, "ilisite.xml")
 
             # download ilimodels.xml
             download_file(
@@ -151,6 +168,13 @@ class IliCache(QObject):
                 ),
             )
 
+    @classmethod
+    def clear_cache(cls):
+        if not QDir().exists(cls.CACHE_PATH):
+            return
+
+        shutil.rmtree(cls.CACHE_PATH, ignore_errors=False, onerror=None)
+
     def _process_ilisite(self, file):
         """
         Parses the ilisite.xml provided in ``file`` and recursively downloads any subidiary sites.
@@ -164,7 +188,7 @@ class IliCache(QObject):
                         file=file, exception=str(e)
                     )
                 ),
-                self.tr("QGIS Model Baker"),
+                self.tr("modelbaker"),
             )
             return
 
@@ -194,7 +218,7 @@ class IliCache(QObject):
                         file=file, exception=str(e)
                     )
                 ),
-                self.tr("QGIS Model Baker"),
+                self.tr("modelbaker"),
             )
             return
 
@@ -240,7 +264,7 @@ class IliCache(QObject):
             reverse=True,
         )
 
-        self.model.set_repositories(self.repositories)
+        self.set_repositories_to_model()
 
     def process_local_ili_folder(self, path):
         """
@@ -256,7 +280,7 @@ class IliCache(QObject):
             models, key=lambda m: m["version"], reverse=True
         )
 
-        self.model.set_repositories(self.repositories)
+        self.set_repositories_to_model()
 
     def process_ili_file(self, ilifile):
         fileModels = list()
@@ -288,7 +312,7 @@ class IliCache(QObject):
                             ilifile=ilifile, exception=str(e)
                         )
                     ),
-                    self.tr("QGIS Model Baker"),
+                    self.tr("modelbaker"),
                 )
                 fileModels = list()
 
@@ -353,7 +377,6 @@ class IliModelItemModel(QStandardItemModel):
 
     def set_repositories(self, repositories):
         self.clear()
-        row = 0
         names = list()
 
         for repository in repositories.values():
@@ -365,13 +388,12 @@ class IliModelItemModel(QStandardItemModel):
 
                 item = QStandardItem()
                 item.setData(model["name"], int(Qt.DisplayRole))
-                item.setData(model["name"], int(Qt.EditRole))
+                item.setData(model["name"], int(Qt.EditRole))  # considered in completer
                 item.setData(model["repository"], int(IliModelItemModel.Roles.ILIREPO))
                 item.setData(model["version"], int(IliModelItemModel.Roles.VERSION))
 
                 names.append(model["name"])
                 self.appendRow(item)
-                row += 1
 
 
 class ModelCompleterDelegate(QItemDelegate):
@@ -423,23 +445,26 @@ class IliDataCache(IliCache):
     file_download_failed = pyqtSignal(str, str)
     model_refreshed = pyqtSignal(int)
 
-    def __init__(self, configuration, type="metaconfig", models=None):
+    CACHE_PATH = os.path.expanduser("~/.ilidatacache")
+
+    def __init__(self, configuration, type="metaconfig", models=None, datasources=[]):
         IliCache.__init__(self, configuration)
-        self.cache_path = os.path.expanduser("~/.ilimetaconfigcache")
         self.information_file = "ilidata.xml"
 
         self.model = IliDataItemModel()
         self.model.modelReset.connect(
             lambda: self.model_refreshed.emit(self.model.rowCount())
         )
-
+        self.sorted_model.setSourceModel(self.model)
+        self.sorted_model.sort(0, Qt.AscendingOrder)
         self.filter_models = models.split(";") if models else []
         self.type = type
         self.directories = (
-            self.base_configuration.metaconfig_directories
+            self.base_configuration.ilidata_directories
             if self.base_configuration
             else []
         )
+        self.datasources = datasources
 
     def process_model_directory(self, path):
         # download remote and local repositories
@@ -458,28 +483,29 @@ class IliDataCache(IliCache):
                         file=file, exception=str(e)
                     )
                 ),
-                self.tr("QGIS Model Baker"),
+                self.tr("modelbaker"),
             )
             return
 
         model_code_regex = re.compile("http://codes.interlis.ch/model/(.*)")
         type_code_regex = re.compile("http://codes.interlis.ch/type/(.*)")
-        tool_code_regex = re.compile("http://codes.opengis.ch/(.*)")
+        datasource_code_regex = re.compile(
+            "http://codes.modelbaker.ch/preferredDataSource/(.*)"
+        )
 
         self.repositories[netloc] = list()
-        repo_metaconfigs = list()
+        repo_ilidatafiles = list()
         for repo in root.iter(
             "{http://www.interlis.ch/INTERLIS2.3}DatasetIdx16.DataIndex"
         ):
-            for metaconfig_metadata in repo.findall(
+            for ilidata_metadata in repo.findall(
                 "ili23:DatasetIdx16.DataIndex.DatasetMetadata", self.ns
             ):
-                categories_element = metaconfig_metadata.find(
-                    "ili23:categories", self.ns
-                )
+                categories_element = ilidata_metadata.find("ili23:categories", self.ns)
                 if categories_element is not None:
                     models = []
                     type = ""
+                    datasource = ""
                     for category in categories_element.findall(
                         "ili23:DatasetIdx16.Code_", self.ns
                     ):
@@ -493,15 +519,24 @@ class IliDataCache(IliCache):
                                 )
                             if type_code_regex.search(category_value):
                                 type = type_code_regex.search(category_value).group(1)
-                            if tool_code_regex.search(category_value):
-                                tool_code_regex.search(category_value).group(1)
+                            if datasource_code_regex.search(category_value):
+                                datasource = datasource_code_regex.search(
+                                    category_value
+                                ).group(1)
+
+                    # filter for models and types and if datasources are passed and found in the category, they need to be considered
                     if (
                         not any(model in models for model in self.filter_models)
                         or type != self.type
+                        or (
+                            self.datasources
+                            and datasource
+                            and datasource not in self.datasources
+                        )
                     ):
                         continue
                     title = list()
-                    for title_element in metaconfig_metadata.findall(
+                    for title_element in ilidata_metadata.findall(
                         "ili23:title", self.ns
                     ):
                         for multilingual_text_element in title_element.findall(
@@ -531,8 +566,44 @@ class IliDataCache(IliCache):
                                     }
                                     title.append(title_information)
 
+                    short_description = list()
+
+                    for short_description_element in ilidata_metadata.findall(
+                        "ili23:shortDescription", self.ns
+                    ):
+                        for (
+                            multilingual_text_element
+                        ) in short_description_element.findall(
+                            "ili23:DatasetIdx16.MultilingualMText", self.ns
+                        ):
+                            for (
+                                localised_text_element
+                            ) in multilingual_text_element.findall(
+                                "ili23:LocalisedText", self.ns
+                            ):
+                                for (
+                                    localised_text_element
+                                ) in localised_text_element.findall(
+                                    "ili23:DatasetIdx16.LocalisedMText", self.ns
+                                ):
+                                    short_description_information = {
+                                        "language": self.get_element_text(
+                                            localised_text_element.find(
+                                                "ili23:Language", self.ns
+                                            )
+                                        ),
+                                        "text": self.get_element_text(
+                                            localised_text_element.find(
+                                                "ili23:Text", self.ns
+                                            )
+                                        ),
+                                    }
+                                    short_description.append(
+                                        short_description_information
+                                    )
+
                     model_link_names = []
-                    for basket_element in metaconfig_metadata.findall(
+                    for basket_element in ilidata_metadata.findall(
                         "ili23:baskets", self.ns
                     ):
                         for basket_metadata in basket_element.findall(
@@ -550,7 +621,7 @@ class IliDataCache(IliCache):
                                         )
                                     )
 
-                    for files_element in metaconfig_metadata.findall(
+                    for files_element in ilidata_metadata.findall(
                         "ili23:files", self.ns
                     ):
                         for data_file in files_element.findall(
@@ -566,39 +637,44 @@ class IliDataCache(IliCache):
                                         file.find("ili23:path", self.ns)
                                     )
 
-                                    metaconfig = dict()
-                                    metaconfig["id"] = self.get_element_text(
-                                        metaconfig_metadata.find("ili23:id", self.ns)
+                                    ilidatafile = dict()
+                                    ilidatafile["id"] = self.get_element_text(
+                                        ilidata_metadata.find("ili23:id", self.ns)
                                     )
 
-                                    metaconfig["version"] = self.get_element_text(
-                                        metaconfig_metadata.find(
-                                            "ili23:version", self.ns
-                                        )
+                                    ilidatafile["version"] = self.get_element_text(
+                                        ilidata_metadata.find("ili23:version", self.ns)
                                     )
-                                    metaconfig["owner"] = self.get_element_text(
-                                        metaconfig_metadata.find("ili23:owner", self.ns)
+                                    ilidatafile["owner"] = self.get_element_text(
+                                        ilidata_metadata.find("ili23:owner", self.ns)
                                     )
 
-                                    metaconfig["repository"] = netloc
-                                    metaconfig["url"] = url
-                                    metaconfig["models"] = models
-                                    metaconfig["relative_file_path"] = path
+                                    ilidatafile["repository"] = netloc
+                                    ilidatafile["url"] = url
+                                    ilidatafile["models"] = models
+                                    ilidatafile["datasource"] = datasource
+                                    ilidatafile["relative_file_path"] = path
                                     if title is not None:
-                                        metaconfig["title"] = title
+                                        ilidatafile["title"] = title
                                     else:
-                                        metaconfig["title"] = None
+                                        ilidatafile["title"] = None
+                                    if short_description is not None:
+                                        ilidatafile[
+                                            "short_description"
+                                        ] = short_description
+                                    else:
+                                        ilidatafile["short_description"] = None
 
-                                    metaconfig["model_link_names"] = model_link_names
-                                    repo_metaconfigs.append(metaconfig)
+                                    ilidatafile["model_link_names"] = model_link_names
+                                    repo_ilidatafiles.append(ilidatafile)
 
         self.repositories[netloc] = sorted(
-            repo_metaconfigs,
+            repo_ilidatafiles,
             key=lambda m: m["version"] if m["version"] else "0",
             reverse=True,
         )
 
-        self.model.set_repositories(self.repositories)
+        self.set_repositories_to_model()
 
     def download_file(self, netloc, url, file, dataset_id=None):
         """
@@ -619,7 +695,7 @@ class IliDataCache(IliCache):
                     self.tr("Could not find local file  {}").format(file_url),
                 )
         else:
-            file_path = os.path.join(self.cache_path, netloc, file)
+            file_path = os.path.join(self.CACHE_PATH, netloc, file)
             file_dir = os.path.dirname(file_path)
             os.makedirs(file_dir, exist_ok=True)
 
@@ -650,6 +726,7 @@ class IliDataItemModel(QStandardItemModel):
         ID = Qt.UserRole + 7
         URL = Qt.UserRole + 8
         MODEL_LINKS = Qt.UserRole + 9
+        SHORT_DESCRIPTION = Qt.UserRole + 10
 
         def __int__(self):
             return self.value
@@ -660,7 +737,6 @@ class IliDataItemModel(QStandardItemModel):
     def set_repositories(self, repositories):
         self.beginResetModel()
         self.clear()
-        row = 0
         ids = list()
 
         for repository in repositories.values():
@@ -675,8 +751,16 @@ class IliDataItemModel(QStandardItemModel):
                     # since there is no multilanguage handling we take the first entry
                     display_value = dataitem["title"][0]["text"]
 
+                short_description = None
+                if (
+                    dataitem["short_description"]
+                    and "text" in dataitem["short_description"][0]
+                ):
+                    short_description = dataitem["short_description"][0]["text"]
+
                 item.setData(display_value, int(Qt.DisplayRole))
                 item.setData(display_value, int(Qt.EditRole))
+                item.setData(short_description, int(Qt.ToolTipRole))
                 item.setData(dataitem["id"], int(IliDataItemModel.Roles.ID))
                 item.setData(
                     dataitem["repository"], int(IliDataItemModel.Roles.ILIREPO)
@@ -690,6 +774,9 @@ class IliDataItemModel(QStandardItemModel):
                 item.setData(dataitem["owner"], int(IliDataItemModel.Roles.OWNER))
                 item.setData(dataitem["title"], int(IliDataItemModel.Roles.TITLE))
                 item.setData(dataitem["url"], int(IliDataItemModel.Roles.URL))
+                item.setData(
+                    short_description, int(IliDataItemModel.Roles.SHORT_DESCRIPTION)
+                )
 
                 item.setData(
                     dataitem["model_link_names"],
@@ -698,11 +785,10 @@ class IliDataItemModel(QStandardItemModel):
 
                 ids.append(dataitem["id"])
                 self.appendRow(item)
-                row += 1
         self.endResetModel()
 
 
-class MetaConfigCompleterDelegate(QItemDelegate):
+class IliDataFileCompleterDelegate(QItemDelegate):
     """
     A item delegate for the autocompleter of metaconfig / topping dialogs.
     It shows the source repository (including model) next to the metaconfig id and the owner.
@@ -713,12 +799,12 @@ class MetaConfigCompleterDelegate(QItemDelegate):
         self.widget = QWidget()
         self.widget.setLayout(QGridLayout())
         self.widget.layout().setContentsMargins(2, 0, 0, 0)
-        self.metaconfig_label = QLabel()
-        self.metaconfig_label.setAttribute(Qt.WA_TranslucentBackground)
+        self.ilidatafile_label = QLabel()
+        self.ilidatafile_label.setAttribute(Qt.WA_TranslucentBackground)
         self.repository_label = QLabel()
         self.repository_label.setAlignment(Qt.AlignRight)
-        self.widget.layout().addWidget(self.metaconfig_label, 0, 0)
-        self.widget.layout().addWidget(self.repository_label, 0, 2)
+        self.widget.layout().addWidget(self.ilidatafile_label, 0, 0)
+        self.widget.layout().addWidget(self.repository_label, 0, 1)
 
     def paint(self, painter, option, index):
         option.index = index
@@ -726,16 +812,14 @@ class MetaConfigCompleterDelegate(QItemDelegate):
 
     def drawDisplay(self, painter, option, rect, text):
         repository = option.index.data(int(IliDataItemModel.Roles.ILIREPO))
-        models = option.index.data(int(IliDataItemModel.Roles.MODELS))
-        owner = option.index.data(int(IliDataItemModel.Roles.OWNER))
         display_text = option.index.data(int(Qt.DisplayRole))
 
         self.repository_label.setText(
-            '<font color="#666666"><i>of {owner} with {models} at {repository}</i></font>'.format(
-                owner=owner, models="/".join(models), repository=repository
+            '<font color="#666666"><i>{repository}</i></font>'.format(
+                repository=repository
             )
         )
-        self.metaconfig_label.setText(
+        self.ilidatafile_label.setText(
             "{display_text}".format(display_text=display_text)
         )
         self.widget.setMinimumSize(rect.size())
@@ -752,16 +836,19 @@ class MetaConfigCompleterDelegate(QItemDelegate):
 
 class IliToppingFileCache(IliDataCache):
 
-    download_finished = pyqtSignal()
+    download_finished_and_model_fresh = pyqtSignal()
     """
     meta_netloc is the repository (netloc) of the metaconfiguration file used for file paths in the file_ids
     file_ids can contain ilidata: or file: information
     """
 
+    CACHE_PATH = os.path.expanduser("~/.ilitoppingfilescache")
+
     def __init__(self, configuration, file_ids=None, tool_dir=None):
         IliDataCache.__init__(self, configuration)
-        self.cache_path = os.path.expanduser("~/.ilitoppingfilescache")
         self.model = IliToppingFileItemModel()
+        self.sorted_model.setSourceModel(self.model)
+        self.sorted_model.sort(0, Qt.AscendingOrder)
         self.file_ids = file_ids
         self.tool_dir = (
             tool_dir
@@ -812,14 +899,15 @@ class IliToppingFileCache(IliDataCache):
             repo_files.append(toppingfile)
 
         self.repositories[netloc] = repo_files
-        self.model.set_repositories(self.repositories)
+        self.set_repositories_to_model()
 
     def on_download_status(self, dataset_id):
         # here we could add some more logic
         if dataset_id is not None:
             self.downloaded_files.append(dataset_id)
+        # ensure all the files are downloaded and contained in the model
         if len(self.downloaded_files) == len(self.file_ids) == self.model.rowCount():
-            self.download_finished.emit()
+            self.download_finished_and_model_fresh.emit()
 
     def _process_informationfile(self, file, netloc, url):
         """
@@ -834,7 +922,7 @@ class IliToppingFileCache(IliDataCache):
                         file=file, exception=str(e)
                     )
                 ),
-                self.tr("QGIS Model Baker"),
+                self.tr("modelbaker"),
             )
             return
 
@@ -892,7 +980,7 @@ class IliToppingFileCache(IliDataCache):
             reverse=True,
         )
 
-        self.model.set_repositories(self.repositories)
+        self.set_repositories_to_model()
 
 
 class IliToppingFileItemModel(QStandardItemModel):
@@ -912,7 +1000,6 @@ class IliToppingFileItemModel(QStandardItemModel):
 
     def set_repositories(self, repositories):
         self.clear()
-        row = 0
         ids = list()
 
         for repository in repositories.values():
@@ -945,4 +1032,3 @@ class IliToppingFileItemModel(QStandardItemModel):
 
                 ids.append(toppingfile["id"])
                 self.appendRow(item)
-                row += 1
