@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 /***************************************************************************
                               -------------------
@@ -50,6 +49,7 @@ class ProjectTopping(QObject):
     - layerorder
     - map themes
     - project variables
+    - project properties
     - print layouts
 
     QML style files, QLR layer definition files and the source of a layer can be linked in the YAML file and are exported to the specific folders.
@@ -62,12 +62,12 @@ class ProjectTopping(QObject):
     LAYERSTYLE_TYPE = "layerstyle"
     LAYOUTTEMPLATE_TYPE = "layouttemplate"
 
-    class TreeItemProperties(object):
+    class TreeItemProperties:
         """
         The properties of a node (tree item)
         """
 
-        class StyleItemProperties(object):
+        class StyleItemProperties:
             """
             The properties of a style item of a node style.
             Currently it's only a qmlstylefile. Maybe in future here a style can be defined.
@@ -105,7 +105,7 @@ class ProjectTopping(QObject):
             # the styles can contain multiple style items with StyleItemProperties
             self.styles = {}
 
-    class LayerTreeItem(object):
+    class LayerTreeItem:
         """
         A tree item of the layer tree. Every item contains the properties of a layer and according the ExportSettings passed on parsing the QGIS project.
         """
@@ -428,6 +428,19 @@ class ProjectTopping(QObject):
                 ).variable(variable_key)
                 self[variable_key] = variable_value or None
 
+    class Properties(dict):
+        """
+        A dict object of dict items describing a selection of projet properties
+        Currently we don't use export settings and export them per default.
+        """
+
+        def make_items(self, project: QgsProject):
+            self.clear()
+            if Qgis.QGIS_VERSION_INT < 32600:
+                self["transaction_mode"] = project.autoTransaction()
+            else:
+                self["transaction_mode"] = project.transactionMode().name
+
     class Layouts(dict):
         """
         A dict object of dict items describing a layout with templatefile according to the layout names listed in the ExportSettings passed on parsing the QGIS project.
@@ -496,6 +509,7 @@ class ProjectTopping(QObject):
         self.mapthemes = self.MapThemes()
         self.layerorder = []
         self.variables = self.Variables()
+        self.properties = self.Properties()
         self.layouts = self.Layouts(temporary_toppingfile_dir)
 
     def parse_project(
@@ -528,6 +542,8 @@ class ProjectTopping(QObject):
             self.variables.make_items(project, export_settings)
             # make print layouts
             self.layouts.make_items(project, export_settings)
+            # make properties
+            self.properties.make_items(project)
 
             self.stdout.emit(
                 self.tr("QGIS project map themes parsed with export settings."),
@@ -586,6 +602,7 @@ class ProjectTopping(QObject):
         Gets the layerorder as a list.
         Gets the mapthemes as a dict.
         Gets the variables as a dict.
+        Gets the properties as a dict.
         Gets the layouts as a dict.
         And it generates and stores the toppingfiles according th the Target.
         """
@@ -599,6 +616,9 @@ class ProjectTopping(QObject):
         variables_dict = dict(self.variables)
         if variables_dict:
             projecttopping_dict["variables"] = variables_dict
+        properties_dict = dict(self.properties)
+        if properties_dict:
+            projecttopping_dict["properties"] = properties_dict
         layouts_item_dict = self.layouts.item_dict(target)
         if layouts_item_dict:
             projecttopping_dict["layouts"] = layouts_item_dict
