@@ -1,7 +1,7 @@
 import os
 import pathlib
 
-from qgis.PyQt.QtCore import Qt
+from qgis.PyQt.QtCore import QEventLoop, Qt, QTimer
 from qgis.testing import unittest
 
 from modelbaker.iliwrapper.ili2dbconfig import BaseConfiguration
@@ -59,7 +59,9 @@ class IliCacheTest(unittest.TestCase):
         ilicache = IliCache([])
         with self.assertRaises(RuntimeError):
             ilicache.parse_ili_file(
-                os.path.join(test_path, "testdata", "ilimodels", "RoadsInvalid.ili"),
+                os.path.join(
+                    test_path, "testdata", "invalid_ilimodels", "RoadsInvalid.ili"
+                ),
                 "utf-8",
             )
 
@@ -75,70 +77,70 @@ class IliCacheTest(unittest.TestCase):
             e["name"] for e in next(elem for elem in ilicache.repositories.values())
         }
         expected_models = {
-                "IliRepository09",
-                "IliRepository09",
-                "IliSite09",
-                "IlisMeta07",
-                "AbstractSymbology",
-                "CoordSys",
-                "RoadsExdm2ben_10",
-                "RoadsExdm2ien_10",
-                "RoadsExgm2ien_10",
-                "StandardSymbology",
-                "Time",
-                "Units",
-                "AbstractSymbology",
-                "CoordSys",
-                "RoadsExdm2ben",
-                "RoadsExdm2ien",
-                "RoadsExgm2ien",
-                "StandardSymbology",
-                "Time",
-                "Units",
-                "GM03Core",
-                "GM03Comprehensive",
-                "CodeISO",
-                "GM03_2Core",
-                "GM03_2Comprehensive",
-                "CodeISO",
-                "GM03_2_1Core",
-                "GM03_2_1Comprehensive",
-                "IlisMeta07",
-                "Units",
-                "IlisMeta07",
-                "IliRepository09",
-                "StandardSymbology",
-                "StandardSymbology",
-                "GM03Core",
-                "GM03_2Core",
-                "GM03_2_1Core",
-                "CoordSys",
-                "INTERLIS_ext",
-                "StandardSymbology",
-                "INTERLIS_ext",
-                "IliVErrors",
-                "RoadsExdm2ien_10",
-                "RoadsExdm2ien",
-                "CoordSys",
-                "StandardSymbology",
-                "Units",
-                "Time",
-                "Time",
-                "Base",
-                "Base_f",
-                "SIA405_Base",
-                "SIA405_Base_f",
-                "Base_LV95",
-                "Base_f_MN95",
-                "SIA405_Base_LV95",
-                "SIA405_Base_f_MN95",
-                "Math",
-                "Text",
-                "DatasetIdx16",
-                "AbstractSymbology",
-                "AbstractSymbology",
-                "Time",
-                "IliRepository20",
+            "IliRepository09",
+            "IliRepository09",
+            "IliSite09",
+            "IlisMeta07",
+            "AbstractSymbology",
+            "CoordSys",
+            "RoadsExdm2ben_10",
+            "RoadsExdm2ien_10",
+            "RoadsExgm2ien_10",
+            "StandardSymbology",
+            "Time",
+            "Units",
+            "AbstractSymbology",
+            "CoordSys",
+            "RoadsExdm2ben",
+            "RoadsExdm2ien",
+            "RoadsExgm2ien",
+            "StandardSymbology",
+            "Time",
+            "Units",
+            "GM03Core",
+            "GM03Comprehensive",
+            "CodeISO",
+            "GM03_2Core",
+            "GM03_2Comprehensive",
+            "CodeISO",
+            "GM03_2_1Core",
+            "GM03_2_1Comprehensive",
+            "IlisMeta07",
+            "Units",
+            "IlisMeta07",
+            "IliRepository09",
+            "StandardSymbology",
+            "StandardSymbology",
+            "GM03Core",
+            "GM03_2Core",
+            "GM03_2_1Core",
+            "CoordSys",
+            "INTERLIS_ext",
+            "StandardSymbology",
+            "INTERLIS_ext",
+            "IliVErrors",
+            "RoadsExdm2ien_10",
+            "RoadsExdm2ien",
+            "CoordSys",
+            "StandardSymbology",
+            "Units",
+            "Time",
+            "Time",
+            "Base",
+            "Base_f",
+            "SIA405_Base",
+            "SIA405_Base_f",
+            "Base_LV95",
+            "Base_f_MN95",
+            "SIA405_Base_LV95",
+            "SIA405_Base_f_MN95",
+            "Math",
+            "Text",
+            "DatasetIdx16",
+            "AbstractSymbology",
+            "AbstractSymbology",
+            "Time",
+            "IliRepository20",
         }
         assert models == expected_models
 
@@ -154,14 +156,14 @@ class IliCacheTest(unittest.TestCase):
             e["name"] for e in next(elem for elem in ilicache.repositories.values())
         }
         expected_models = {
-                "AbstractSymbology",
-                "CoordSys",
-                "RoadsExdm2ben",
-                "RoadsExdm2ien",
-                "RoadsExgm2ien",
-                "StandardSymbology",
-                "Time",
-                "Units",
+            "AbstractSymbology",
+            "CoordSys",
+            "RoadsExdm2ben",
+            "RoadsExdm2ien",
+            "RoadsExgm2ien",
+            "StandardSymbology",
+            "Time",
+            "Units",
         }
         assert models == expected_models
 
@@ -178,44 +180,44 @@ class IliCacheTest(unittest.TestCase):
             in ilicache.repositories.keys()
         )
         models = {
-                model["name"]
-                for model in [
-                    e for values in ilicache.repositories.values() for e in values
-                ]
+            model["name"]
+            for model in [
+                e for values in ilicache.repositories.values() for e in values
+            ]
         }
         expected_models_of_ilimodels_xml = {
-                "AbstractSymbology",
-                "CoordSys",
-                "RoadsExdm2ben",
-                "RoadsExdm2ien",
-                "RoadsExgm2ien",
-                "StandardSymbology",
-                "Time",
-                "Units",
+            "AbstractSymbology",
+            "CoordSys",
+            "RoadsExdm2ben",
+            "RoadsExdm2ien",
+            "RoadsExgm2ien",
+            "StandardSymbology",
+            "Time",
+            "Units",
         }
         expected_models_of_local_ili_files = {
-                "KbS_Basis_V1_4",
-                "KbS_LV03_V1_4",
-                "KbS_LV95_V1_4",
-                "RoadsSimple",
-                "Abfallsammelstellen_ZEBA_LV03_V1",
-                "Abfallsammelstellen_ZEBA_LV95_V1",
-                "DictionariesCH_V1",
-                "GeometryCHLV03_V1",
-                "S",
-                "AdministrativeUnitsCH_V1",
-                "CHAdminCodes_V1",
-                "Localisation_V1",
-                "LE",
-                "LocalisationCH_V1",
-                "Dictionaries_V1",
-                "CatalogueObjectTrees_V1",
-                "AdministrativeUnits_V1",
-                "GeometryCHLV95_V1",
-                "InternationalCodes_V1",
-                "CatalogueObjects_V1",
-                "PlanerischerGewaesserschutz_LV95_V1_1",
-                "PlanerischerGewaesserschutz_LV03_V1_1",
+            "KbS_Basis_V1_4",
+            "KbS_LV03_V1_4",
+            "KbS_LV95_V1_4",
+            "RoadsSimple",
+            "Abfallsammelstellen_ZEBA_LV03_V1",
+            "Abfallsammelstellen_ZEBA_LV95_V1",
+            "DictionariesCH_V1",
+            "GeometryCHLV03_V1",
+            "S",
+            "AdministrativeUnitsCH_V1",
+            "CHAdminCodes_V1",
+            "Localisation_V1",
+            "LE",
+            "LocalisationCH_V1",
+            "Dictionaries_V1",
+            "CatalogueObjectTrees_V1",
+            "AdministrativeUnits_V1",
+            "GeometryCHLV95_V1",
+            "InternationalCodes_V1",
+            "CatalogueObjects_V1",
+            "PlanerischerGewaesserschutz_LV95_V1_1",
+            "PlanerischerGewaesserschutz_LV03_V1_1",
         }
 
         assert models == set.union(
@@ -238,44 +240,44 @@ class IliCacheTest(unittest.TestCase):
             in ilicache.repositories.keys()
         )
         models = {
-                model["name"]
-                for model in [
-                    e for values in ilicache.repositories.values() for e in values
-                ]
+            model["name"]
+            for model in [
+                e for values in ilicache.repositories.values() for e in values
+            ]
         }
         expected_models_of_ilimodels_xml = {
-                "AbstractSymbology",
-                "CoordSys",
-                "RoadsExdm2ben",
-                "RoadsExdm2ien",
-                "RoadsExgm2ien",
-                "StandardSymbology",
-                "Time",
-                "Units",
+            "AbstractSymbology",
+            "CoordSys",
+            "RoadsExdm2ben",
+            "RoadsExdm2ien",
+            "RoadsExgm2ien",
+            "StandardSymbology",
+            "Time",
+            "Units",
         }
         expected_models_of_local_ili_files = {
-                "KbS_Basis_V1_4",
-                "KbS_LV03_V1_4",
-                "KbS_LV95_V1_4",
-                "RoadsSimple",
-                "Abfallsammelstellen_ZEBA_LV03_V1",
-                "Abfallsammelstellen_ZEBA_LV95_V1",
-                "DictionariesCH_V1",
-                "GeometryCHLV03_V1",
-                "S",
-                "AdministrativeUnitsCH_V1",
-                "CHAdminCodes_V1",
-                "Localisation_V1",
-                "LE",
-                "LocalisationCH_V1",
-                "Dictionaries_V1",
-                "CatalogueObjectTrees_V1",
-                "AdministrativeUnits_V1",
-                "GeometryCHLV95_V1",
-                "InternationalCodes_V1",
-                "CatalogueObjects_V1",
-                "PlanerischerGewaesserschutz_LV95_V1_1",
-                "PlanerischerGewaesserschutz_LV03_V1_1",
+            "KbS_Basis_V1_4",
+            "KbS_LV03_V1_4",
+            "KbS_LV95_V1_4",
+            "RoadsSimple",
+            "Abfallsammelstellen_ZEBA_LV03_V1",
+            "Abfallsammelstellen_ZEBA_LV95_V1",
+            "DictionariesCH_V1",
+            "GeometryCHLV03_V1",
+            "S",
+            "AdministrativeUnitsCH_V1",
+            "CHAdminCodes_V1",
+            "Localisation_V1",
+            "LE",
+            "LocalisationCH_V1",
+            "Dictionaries_V1",
+            "CatalogueObjectTrees_V1",
+            "AdministrativeUnits_V1",
+            "GeometryCHLV95_V1",
+            "InternationalCodes_V1",
+            "CatalogueObjects_V1",
+            "PlanerischerGewaesserschutz_LV95_V1_1",
+            "PlanerischerGewaesserschutz_LV03_V1_1",
         }
         assert models == set.union(
             expected_models_of_ilimodels_xml, expected_models_of_local_ili_files
@@ -283,6 +285,58 @@ class IliCacheTest(unittest.TestCase):
 
     def test_ilidata_xml_parser_metaconfig_kbs(self):
         # find kbs metaconfig file according to the model(s) with direct ilidata.xml scan
+        # pass gpkg as datasource find the ones with gpkg-datasource-category and the ones without datasource-category
+        ilimetaconfigcache = IliDataCache(
+            configuration=None, models="KbS_LV95_V1_4", datasources=["gpkg"]
+        )
+        ilimetaconfigcache._process_informationfile(
+            os.path.join(
+                test_path, "testdata", "ilirepo", "usabilityhub", "ilidata.xml"
+            ),
+            "test_repo",
+            os.path.join(test_path, "testdata", "ilirepo", "usabilityhub"),
+        )
+        assert "test_repo" in ilimetaconfigcache.repositories.keys()
+        metaconfigs = {
+            e["id"]
+            for e in next(elem for elem in ilimetaconfigcache.repositories.values())
+        }
+        expected_metaconfigs = {
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-technical",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_gpkg_localfiletest",  # gpkg as preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-wrong",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_gpkg",  # no preferred datasource
+        }
+        assert metaconfigs == expected_metaconfigs
+
+        # pass pg as datasource find the ones with pg-datasource-category and the ones without datasource-category
+        ilimetaconfigcache = IliDataCache(
+            configuration=None, models="KbS_LV95_V1_4", datasources=["pg"]
+        )
+        ilimetaconfigcache._process_informationfile(
+            os.path.join(
+                test_path, "testdata", "ilirepo", "usabilityhub", "ilidata.xml"
+            ),
+            "test_repo",
+            os.path.join(test_path, "testdata", "ilirepo", "usabilityhub"),
+        )
+        assert "test_repo" in ilimetaconfigcache.repositories.keys()
+        metaconfigs = {
+            e["id"]
+            for e in next(elem for elem in ilimetaconfigcache.repositories.values())
+        }
+        expected_metaconfigs = {
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-technical",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_localfiletest",  # pg as preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_ili2db",  # pg as preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-wrong",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_gpkg",  # no preferred datasource
+        }
+        assert metaconfigs == expected_metaconfigs
+
+        # don't pass any datasource, means find all
         ilimetaconfigcache = IliDataCache(configuration=None, models="KbS_LV95_V1_4")
         ilimetaconfigcache._process_informationfile(
             os.path.join(
@@ -293,22 +347,24 @@ class IliCacheTest(unittest.TestCase):
         )
         assert "test_repo" in ilimetaconfigcache.repositories.keys()
         metaconfigs = {
-                e["id"]
-                for e in next(elem for elem in ilimetaconfigcache.repositories.values())
+            e["id"]
+            for e in next(elem for elem in ilimetaconfigcache.repositories.values())
         }
         expected_metaconfigs = {
-            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-technical",
-            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0",
-            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_localfiletest",
-            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_gpkg_localfiletest",
-            "ch.opengis.ili.config.KbS_LV95_V1_4_ili2db",
-            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-wrong",
-            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_gpkg",
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-technical",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_localfiletest",  # pg as preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_gpkg_localfiletest",  # gpkg as preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_ili2db",  # pg as preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0-wrong",  # no preferred datasource
+            "ch.opengis.ili.config.KbS_LV95_V1_4_config_V1_0_gpkg",  # no preferred datasource
         }
         assert metaconfigs == expected_metaconfigs
 
         ilimetaconfigcache_model = ilimetaconfigcache.model
 
+        self._sleep(ilimetaconfigcache_model)
+
         matches_on_id = ilimetaconfigcache_model.match(
             ilimetaconfigcache_model.index(0, 0),
             int(IliDataItemModel.Roles.ID),
@@ -376,8 +432,8 @@ class IliCacheTest(unittest.TestCase):
         )
 
         metaconfigs = {
-                e["id"]
-                for e in next(elem for elem in ilimetaconfigcache.repositories.values())
+            e["id"]
+            for e in next(elem for elem in ilimetaconfigcache.repositories.values())
         }
         expected_metaconfigs = {
             "ch.opengis.ili.config.PlanerischerGewaesserschutz_config",
@@ -387,6 +443,8 @@ class IliCacheTest(unittest.TestCase):
 
         ilimetaconfigcache_model = ilimetaconfigcache.model
 
+        self._sleep(ilimetaconfigcache_model)
+
         matches_on_id = ilimetaconfigcache_model.match(
             ilimetaconfigcache_model.index(0, 0),
             int(IliDataItemModel.Roles.ID),
@@ -446,10 +504,8 @@ class IliCacheTest(unittest.TestCase):
         )
         assert "test_repo" in ilireferencedatacache.repositories.keys()
         referencedata = {
-                e["id"]
-                for e in next(
-                    elem for elem in ilireferencedatacache.repositories.values()
-                )
+            e["id"]
+            for e in next(elem for elem in ilireferencedatacache.repositories.values())
         }
         expected_referencedata = {
             "ch.opengis.ili.catalogue.PlanerischerGewaesserschutz_Codetexte_V1_1",
@@ -458,6 +514,8 @@ class IliCacheTest(unittest.TestCase):
         }
         assert referencedata == expected_referencedata
 
+        self._sleep(ilireferencedatacache.model)
+
         linked_model_list = []
         for r in range(ilireferencedatacache.model.rowCount()):
             if ilireferencedatacache.model.item(r).data(
@@ -501,10 +559,8 @@ class IliCacheTest(unittest.TestCase):
         )
 
         referencedata = {
-                e["id"]
-                for e in next(
-                    elem for elem in ilireferencedatacache.repositories.values()
-                )
+            e["id"]
+            for e in next(elem for elem in ilireferencedatacache.repositories.values())
         }
         expected_referencedata = {
             "ch.opengis.ili.catalogue.PlanerischerGewaesserschutz_Codetexte_V1_1",
@@ -513,6 +569,8 @@ class IliCacheTest(unittest.TestCase):
         }
         assert referencedata == expected_referencedata
 
+        self._sleep(ilireferencedatacache.model)
+
         linked_model_list = []
         for r in range(ilireferencedatacache.model.rowCount()):
             if ilireferencedatacache.model.item(r).data(
@@ -556,10 +614,8 @@ class IliCacheTest(unittest.TestCase):
         )
         assert "test_repo" in ilitoppingfilecache.repositories.keys()
         files = {
-                e["relative_file_path"]
-                for e in next(
-                    elem for elem in ilitoppingfilecache.repositories.values()
-                )
+            e["relative_file_path"]
+            for e in next(elem for elem in ilitoppingfilecache.repositories.values())
         }
         expected_files = {
             "layerstyle/opengisch_KbS_LV95_V1_4_005_parzellenidentifikation.qml",
@@ -571,6 +627,8 @@ class IliCacheTest(unittest.TestCase):
 
         ilitoppingfilecache_model = ilitoppingfilecache.model
 
+        self._sleep(ilitoppingfilecache_model)
+
         matches_on_id = ilitoppingfilecache_model.match(
             ilitoppingfilecache_model.index(0, 0),
             Qt.DisplayRole,
@@ -635,10 +693,8 @@ class IliCacheTest(unittest.TestCase):
         assert "local_files" in ilitoppingfilecache.repositories.keys()
 
         files = {
-                e["relative_file_path"]
-                for e in next(
-                    elem for elem in ilitoppingfilecache.repositories.values()
-                )
+            e["relative_file_path"]
+            for e in next(elem for elem in ilitoppingfilecache.repositories.values())
         }
         expected_files = {
             "layerstyle/opengisch_KbS_LV95_V1_4_005_parzellenidentifikation.qml",
@@ -673,10 +729,8 @@ class IliCacheTest(unittest.TestCase):
         assert "local_files" in ilitoppingfilecache.repositories.keys()
 
         files = {
-                e["relative_file_path"]
-                for e in next(
-                    elem for elem in ilitoppingfilecache.repositories.values()
-                )
+            e["relative_file_path"]
+            for e in next(elem for elem in ilitoppingfilecache.repositories.values())
         }
         expected_files = {
             "layerstyle/opengisch_KbS_LV95_V1_4_005_parzellenidentifikation.qml",
@@ -715,9 +769,19 @@ class IliCacheTest(unittest.TestCase):
         )
         assert "test_repo" in ilimetaconfigcache.repositories.keys()
         metaconfigs = {
-                e["id"]
-                for e in next(elem for elem in ilimetaconfigcache.repositories.values())
+            e["id"]
+            for e in next(elem for elem in ilimetaconfigcache.repositories.values())
         }
         # not finding invalid metaconfig but the one with none as id
         expected_metaconfigs = {None, "ch.opengis.ili.config.valid"}
         assert metaconfigs == expected_metaconfigs
+
+    def _sleep(self, model=None, mili=10000):
+        loop = QEventLoop()
+        timer = QTimer()
+        timer.setSingleShot(True)
+        timer.timeout.connect(lambda: loop.quit())
+        timer.start(mili)
+        if model:
+            model.modelReset.connect(lambda: loop.quit())
+        loop.exec()
