@@ -1238,7 +1238,9 @@ class TestProjectTopping(unittest.TestCase):
 
         # we wait for the download or we timeout after 30 seconds and we apply what we have
         loop = QEventLoop()
-        topping_file_cache.download_finished.connect(lambda: loop.quit())
+        topping_file_cache.download_finished_and_model_fresh.connect(
+            lambda: loop.quit()
+        )
         timer = QTimer()
         timer.setSingleShot(True)
         timer.timeout.connect(lambda: loop.quit())
@@ -1249,6 +1251,10 @@ class TestProjectTopping(unittest.TestCase):
         if len(topping_file_cache.downloaded_files) != len(id_list):
             loop.exec()
 
+        # since the local download is quicker than the delay in the model refresh, we make one here as well
+        timer.start(3000)
+        loop.exec()
+
         return topping_file_cache.model
 
     # that's the same (more or less) like in project_creation_page.py
